@@ -2,9 +2,11 @@
 #include "logging_private.h"
 #include "ControlSocket.h"
 #include <idna.h>
-#include "asynchostresolver.h"
 #include "directorycache.h"
 #include "servercapabilities.h"
+#include "local_filesys.h"
+#include <errno.h>
+#include "proxy.h"
 
 DECLARE_EVENT_TYPE(fzOBTAINLOCK, -1)
 DEFINE_EVENT_TYPE(fzOBTAINLOCK)
@@ -749,9 +751,14 @@ void CControlSocket::UnlockCache()
 	if (iter == m_lockInfoList.end())
 		return;
 
-	iter->lockcount--;
-	if (iter->lockcount)
-		return;
+	wxASSERT(!iter->waiting || iter->lockcount == 0);
+	if (!iter->waiting)
+	{
+		iter->lockcount--;
+		wxASSERT(iter->lockcount >= 0);
+		if (iter->lockcount)
+			return;
+	}
 
 	CServerPath directory = iter->directory;
 	enum locking_reason reason = iter->reason;
@@ -779,7 +786,8 @@ void CControlSocket::UnlockCache()
 
 enum CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 {
-	wxASSERT(m_pCurOpData);
+	if (!m_pCurOpData)
+		return lock_unknown;
 
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end())
@@ -862,21 +870,24 @@ void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
 // ------------------
 
 BEGIN_EVENT_TABLE(CRealControlSocket, CControlSocket)
-	EVT_SOCKET(wxID_ANY, CRealControlSocket::OnSocketEvent)
+	EVT_FZ_SOCKET(wxID_ANY, CRealControlSocket::OnSocketEvent)
 END_EVENT_TABLE()
 
 CRealControlSocket::CRealControlSocket(CFileZillaEnginePrivate *pEngine)
-	: CControlSocket(pEngine), wxSocketClient(wxSOCKET_NOWAIT)
+	: CControlSocket(pEngine), CSocket(this, 0)
 {
 	m_pBackend = new CSocketBackend(this, this);
+	m_pProxyBackend = 0;
 
 	m_pSendBuffer = 0;
 	m_nSendBufferLen = 0;
-	m_onConnectCalled = false;
 }
 
 CRealControlSocket::~CRealControlSocket()
 {
+	Close();
+	if (m_pProxyBackend && m_pProxyBackend != m_pBackend)
+		delete m_pProxyBackend;
 	delete m_pBackend;
 	m_pBackend = 0;
 }
@@ -899,8 +910,11 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 		int numsent = 0;
 		if (m_pBackend->Error())
 		{
-			if (m_pBackend->LastError() != wxSOCKET_WOULDBLOCK)
+			const int error = m_pBackend->LastError();
+			if (error != EAGAIN)
 			{
+				LogMessage(::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error).c_str());
+				LogMessage(::Error, _("Disconnected from server"));
 				DoClose();
 				return false;
 			}
@@ -925,7 +939,7 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 	return true;
 }
 
-void CRealControlSocket::OnSocketEvent(wxSocketEvent &event)
+void CRealControlSocket::OnSocketEvent(CSocketEvent &event)
 {
 	if (!m_pBackend)
 		return;
@@ -933,25 +947,45 @@ void CRealControlSocket::OnSocketEvent(wxSocketEvent &event)
 	if (event.GetId() != m_pBackend->GetId())
 		return;
 
-	switch (event.GetSocketEvent())
+	switch (event.GetType())
 	{
-	case wxSOCKET_CONNECTION:
-		m_onConnectCalled = true;
-		OnConnect();
-		break;
-	case wxSOCKET_INPUT:
-		if (!m_onConnectCalled)
+	case CSocketEvent::hostaddress:
 		{
-			m_onConnectCalled = true;
+			const wxString& address = event.GetData();
+			LogMessage(Status, _("Connecting to %s..."), address.c_str()); 
+		}
+		break;
+	case CSocketEvent::connection_next:
+		if (event.GetError())
+			LogMessage(Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()).c_str()); 
+		break;
+	case CSocketEvent::connection:
+		if (event.GetError())
+		{
+			LogMessage(Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()).c_str()); 
+			OnClose(event.GetError());
+		}
+		else
+		{
+			if (m_pProxyBackend && !m_pProxyBackend->Detached())
+			{
+				m_pProxyBackend->Detach();
+				m_pBackend = new CSocketBackend(this, this);
+			}
 			OnConnect();
 		}
+		break;
+	case CSocketEvent::read:
 		OnReceive();
 		break;
-	case wxSOCKET_OUTPUT:
+	case CSocketEvent::write:
 		OnSend();
 		break;
-	case wxSOCKET_LOST:
-		OnClose();
+	case CSocketEvent::close:
+		OnClose(event.GetError());
+		break;
+	default:
+		LogMessage(Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
 		break;
 	}
 }
@@ -978,8 +1012,14 @@ void CRealControlSocket::OnSend()
 		m_pBackend->Write(m_pSendBuffer, m_nSendBufferLen);
 		if (m_pBackend->Error())
 		{
-			if (m_pBackend->LastError() != wxSOCKET_WOULDBLOCK)
+			const int error = m_pBackend->LastError();
+			if (error != EAGAIN)
+			{
+				LogMessage(::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error).c_str());
+				if (GetCurrentCommandId() != cmd_connect)
+					LogMessage(::Error, _("Disconnected from server"));
 				DoClose();
+			}
 			return;
 		}
 
@@ -1005,12 +1045,17 @@ void CRealControlSocket::OnSend()
 	}
 }
 
-void CRealControlSocket::OnClose()
+void CRealControlSocket::OnClose(int error)
 {
-	LogMessage(Debug_Verbose, _T("CRealControlSocket::OnClose()"));
+	LogMessage(Debug_Verbose, _T("CRealControlSocket::OnClose(%d)"), error);
 
 	if (GetCurrentCommandId() != cmd_connect)
-		LogMessage(::Error, _("Disconnected from server"));
+	{
+		if (!error)
+			LogMessage(::Error, _("Connection closed by server"));
+		else
+			LogMessage(::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error).c_str());
+	}
 	DoClose();
 }
 
@@ -1025,64 +1070,66 @@ int CRealControlSocket::Connect(const CServer &server)
 		delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
 
-	CConnectOpData* pData;
-	if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
-		pData = 0;
-	else
-		pData = static_cast<CConnectOpData *>(m_pCurOpData);
+	// International domain names
+	m_pCurrentServer->SetHost(ConvertDomainName(server.GetHost()), server.GetPort());
 
-	const wxString& host = pData ? pData->host : server.GetHost();
-	if (!IsIpAddress(host))
-	{
-		LogMessage(Status, _("Resolving IP-Address for %s"), host.c_str());
-		CAsyncHostResolver *resolver = new CAsyncHostResolver(m_pEngine, ConvertDomainName(host));
-		m_pEngine->AddNewAsyncHostResolver(resolver);
-
-		resolver->Create();
-		resolver->Run();
-	}
-	else
-	{
-		wxIPV4address addr;
-		addr.Hostname(host);
-		return ContinueConnect(&addr);
-	}
-
-	return FZ_REPLY_WOULDBLOCK;
+	return ContinueConnect();
 }
 
-int CRealControlSocket::ContinueConnect(const wxIPV4address *address)
+int CRealControlSocket::ContinueConnect()
 {
-	LogMessage(__TFILE__, __LINE__, this, Debug_Verbose, _T("CRealControlSocket::ContinueConnect(%p) m_pEngine=%p"), address, m_pEngine);
-	if (GetCurrentCommandId() != cmd_connect ||
-		!m_pCurrentServer)
-	{
-		LogMessage(Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
-		return DoClose(FZ_REPLY_INTERNALERROR);
-	}
+	wxString host;
+	unsigned int port = 0;
 
-	if (!address)
+	const int proxy_type = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_TYPE);
+	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy())
 	{
-		LogMessage(::Error, _("Invalid hostname or host not found"));
-		return DoClose(FZ_REPLY_ERROR | FZ_REPLY_CRITICALERROR);
-	}
+		LogMessage(::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost().c_str());
 
-	CConnectOpData* pData;
-	if (!m_pCurOpData || m_pCurOpData->opId != cmd_connect)
-		pData = 0;
+		host = m_pEngine->GetOptions()->GetOption(OPTION_PROXY_HOST);
+		port = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_PORT);
+
+		delete m_pBackend;
+		m_pProxyBackend = new CProxySocket(this, this, this);
+		m_pBackend = m_pProxyBackend;
+		int res = m_pProxyBackend->Handshake((enum CProxySocket::ProxyType)proxy_type,
+											 m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(),
+											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_USER),
+											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_PASS));
+
+		if (res != EINPROGRESS)
+		{
+			LogMessage(::Error, _("Could not start proxy handshake: %s"), CSocket::GetErrorDescription(res).c_str());
+			DoClose();
+			return FZ_REPLY_ERROR;
+		}
+	}
 	else
-		pData = static_cast<CConnectOpData *>(m_pCurOpData);
+	{
+		if (m_pCurOpData && m_pCurOpData->opId == cmd_connect)
+		{
+			CConnectOpData* pData = (CConnectOpData*)m_pCurOpData;
+			host = ConvertDomainName(pData->host);
+			port = pData->port;
+		}
+		if (host == _T(""))
+		{
+			host = m_pCurrentServer->GetHost();
+			port = m_pCurrentServer->GetPort();
+		}
+	}
+	if (!IsIpAddress(host))
+		LogMessage(Status, _("Resolving address of %s"), host.c_str());
 
-	const unsigned int port = pData ? pData->port : m_pCurrentServer->GetPort();
-	LogMessage(Status, _("Connecting to %s:%d..."), address->IPAddress().c_str(), port);
+	int res = CSocket::Connect(host, port);
 
-	wxIPV4address addr = *address;
-	addr.Service(port);
-
-	bool res = wxSocketClient::Connect(addr, false);
-
-	if (!res && LastError() != wxSOCKET_WOULDBLOCK)
-		return DoClose();
+	// Treat success same as EINPROGRESS, we wait for connect notification in any case
+	if (res && res != EINPROGRESS)
+	{
+		LogMessage(::Error, _("Could not connect to server: %s"), CSocket::GetErrorDescription(res).c_str());
+		DoClose();
+		return FZ_REPLY_ERROR;
+	}
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -1105,26 +1152,161 @@ void CRealControlSocket::ResetSocket()
 		m_nSendBufferLen = 0;
 	}
 
-	m_onConnectCalled = false;
-
+	if (m_pProxyBackend)
+	{
+		if (m_pProxyBackend != m_pBackend)
+			delete m_pProxyBackend;
+		m_pProxyBackend = 0;
+	}
 	delete m_pBackend;
 	m_pBackend = 0;
 }
 
-wxString CRealControlSocket::GetLocalIP() const
+bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNotification)
 {
-	wxIPV4address addr;
-	if (!GetLocal(addr))
-		return _T("");
+	wxASSERT(pFileExistsNotification);
 
-	return addr.IPAddress();
-}
+	if (!m_pCurOpData || m_pCurOpData->opId != cmd_transfer)
+	{
+		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
+		return false;
+	}
 
-wxString CRealControlSocket::GetPeerIP() const
-{
-	wxIPV4address addr;
-	if (!GetPeer(addr))
-		return _T("");
+	CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
 
-	return addr.IPAddress();
+	switch (pFileExistsNotification->overwriteAction)
+	{
+	case CFileExistsNotification::overwrite:
+		SendNextCommand();
+		break;
+	case CFileExistsNotification::overwriteNewer:
+		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+			SendNextCommand();
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else
+		{
+			if (pData->download)
+			{
+				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			}
+			else
+			{
+				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			}
+			ResetOperation(FZ_REPLY_OK);
+		}
+		break;
+	case CFileExistsNotification::overwriteSize:
+		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
+		Second compare flags the remaining case in which we need to send command : both size unknown */
+		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
+			SendNextCommand();
+		else
+		{
+			if (pData->download)
+			{
+				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			}
+			else
+			{
+				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			}
+			ResetOperation(FZ_REPLY_OK);
+		}
+		break;
+	case CFileExistsNotification::overwriteSizeOrNewer:
+		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+			SendNextCommand();
+		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
+		Second compare flags the remaining case in which we need to send command : both size unknown */
+		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
+			SendNextCommand();
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+			SendNextCommand();
+		else
+		{
+			if (pData->download)
+			{
+				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			}
+			else
+			{
+				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			}
+			ResetOperation(FZ_REPLY_OK);
+		}
+		break;
+	case CFileExistsNotification::resume:
+		if (pData->download && pData->localFileSize != -1)
+			pData->resume = true;
+		else if (!pData->download && pData->remoteFileSize != -1)
+			pData->resume = true;
+		SendNextCommand();
+		break;
+	case CFileExistsNotification::rename:
+		if (pData->download)
+		{
+			wxFileName fn = pData->localFile;
+			fn.SetFullName(pFileExistsNotification->newName);
+			pData->localFile = fn.GetFullPath();
+
+			wxLongLong size;
+			bool isLink;
+			if (CLocalFileSystem::GetFileInfo(pData->localFile, isLink, &size, 0, 0) == CLocalFileSystem::file)
+				pData->localFileSize = size.GetValue();
+			else
+				pData->localFileSize = -1;
+
+			if (CheckOverwriteFile() == FZ_REPLY_OK)
+				SendNextCommand();
+		}
+		else
+		{
+			pData->remoteFile = pFileExistsNotification->newName;
+
+			CDirectoryCache cache;
+
+			CDirentry entry;
+			bool dir_did_exist;
+			bool matched_case;
+			if (cache.LookupFile(entry, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, pData->remoteFile, dir_did_exist, matched_case) &&
+				matched_case)
+			{
+				wxLongLong size = entry.size;
+				pData->remoteFileSize = size.GetLo() + ((wxFileOffset)size.GetHi() << 32);
+
+				if (CheckOverwriteFile() != FZ_REPLY_OK)
+					break;
+			}
+
+			SendNextCommand();
+		}
+		break;
+	case CFileExistsNotification::skip:
+		if (pData->download)
+		{
+			wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+			LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+		}
+		else
+		{
+			LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+		}
+		ResetOperation(FZ_REPLY_OK);
+		break;
+	default:
+		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown file exists action: %d"), pFileExistsNotification->overwriteAction);
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return false;
+	}
+
+	return true;
 }
