@@ -1,24 +1,30 @@
 #include "FileZilla.h"
 #include "backend.h"
+#include "socket.h"
+#include <errno.h>
 
 int CBackend::m_nextId = 0;
 
 CBackend::CBackend(wxEvtHandler* pEvtHandler) : m_pEvtHandler(pEvtHandler)
 {
-	m_Id = m_nextId++;
-	if (m_nextId < 0)
-		m_nextId = 0;
+	m_Id = GetNextId();
 }
 
-CSocketBackend::CSocketBackend(wxEvtHandler* pEvtHandler, wxSocketBase* pSocket) : CBackend(pEvtHandler), m_pSocket(pSocket)
+int CBackend::GetNextId()
+{
+	const int id = m_nextId++;
+	if (m_nextId < 0)
+		m_nextId = 0;
+	return id;
+}
+
+CSocketBackend::CSocketBackend(wxEvtHandler* pEvtHandler, CSocket* pSocket) : CBackend(pEvtHandler), m_pSocket(pSocket)
 {
 	m_error = false;
 	m_lastCount = 0;
 	m_lastError = 0;
 
-	m_pSocket->SetEventHandler(*pEvtHandler, GetId());
-	m_pSocket->SetNotify(wxSOCKET_CONNECTION_FLAG | wxSOCKET_INPUT_FLAG | wxSOCKET_OUTPUT_FLAG | wxSOCKET_LOST_FLAG);
-	m_pSocket->Notify(true);
+	m_pSocket->SetEventHandler(pEvtHandler, GetId());
 
 	CRateLimiter* pRateLimiter = CRateLimiter::Get();
 	if (pRateLimiter)
@@ -27,21 +33,13 @@ CSocketBackend::CSocketBackend(wxEvtHandler* pEvtHandler, wxSocketBase* pSocket)
 
 CSocketBackend::~CSocketBackend()
 {
-	m_pSocket->Notify(false);
+	m_pSocket->SetEventHandler(0, -1);
 
 	CRateLimiter* pRateLimiter = CRateLimiter::Get();
 	if (pRateLimiter)
 		pRateLimiter->RemoveObject(this);
 }
 
-void CSocketBackend::UpdateResults()
-{
-	if ((m_error = m_pSocket->Error()))
-		m_lastError = m_pSocket->LastError();
-	else
-		m_lastCount = m_pSocket->LastCount();
-}
-
 void CSocketBackend::Write(const void *buffer, unsigned int len)
 {
 	wxLongLong max = GetAvailableBytes(CRateLimiter::outbound);
@@ -49,14 +47,14 @@ void CSocketBackend::Write(const void *buffer, unsigned int len)
 	{
 		Wait(CRateLimiter::outbound);
 		m_error = true;
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		return;
 	}
 	else if (max > 0 && max < len)
 		len = max.GetLo();
 
-	m_pSocket->Write(buffer, len);
-	UpdateResults();
+	m_lastCount = m_pSocket->Write(buffer, len, m_lastError);
+	m_error = m_lastCount == -1;
 
 	if (!m_error && max != -1)
 		UpdateUsage(CRateLimiter::outbound, m_lastCount);
@@ -69,14 +67,14 @@ void CSocketBackend::Read(void *buffer, unsigned int len)
 	{
 		Wait(CRateLimiter::inbound);
 		m_error = true;
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		return;
 	}
 	else if (max > 0 && max < len)
 		len = max.GetLo();
 
-	m_pSocket->Read(buffer, len);
-	UpdateResults();
+	m_lastCount = m_pSocket->Read(buffer, len, m_lastError);
+	m_error = m_lastCount == -1;
 
 	if (!m_error && max != -1)
 		UpdateUsage(CRateLimiter::inbound, m_lastCount);
@@ -84,17 +82,20 @@ void CSocketBackend::Read(void *buffer, unsigned int len)
 
 void CSocketBackend::Peek(void *buffer, unsigned int len)
 {
-	m_pSocket->Peek(buffer, len);
-	UpdateResults();
+	m_lastCount = m_pSocket->Peek(buffer, len, m_lastError);
+	m_error = m_lastCount == -1;
 }
 
 void CSocketBackend::OnRateAvailable(enum CRateLimiter::rate_direction direction)
 {
-	wxSocketEvent evt;
-	evt.SetId(GetId());
 	if (direction == CRateLimiter::outbound)
-		evt.m_event = wxSOCKET_OUTPUT;
+	{
+		CSocketEvent evt(GetId(), CSocketEvent::write);
+		m_pEvtHandler->AddPendingEvent(evt);
+	}
 	else
-		evt.m_event = wxSOCKET_INPUT;
-	wxPostEvent(m_pEvtHandler, evt);
+	{
+		CSocketEvent evt(GetId(), CSocketEvent::read);
+		m_pEvtHandler->AddPendingEvent(evt);
+	}
 }
