@@ -25,6 +25,7 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 {
 	CServerEntry* pServerEntry = CreateServerEntry(server);
 	wxASSERT(pServerEntry);
+	wxASSERT(!parentPath.IsEmpty());
 
 	// First check for existing entry in cache and update if it neccessary
 	for (tCacheIter iter = pServerEntry->cacheList.begin(); iter != pServerEntry->cacheList.end(); iter++)
@@ -244,30 +245,21 @@ bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const
 
 			const CDirectoryListing &listing = cacheEntry.listing;
 
-			bool found = false;
-			for (unsigned int i = 0; i < listing.GetCount(); i++)
+			int i = listing.FindFile_CmpCase(file);
+			if (i != -1)
 			{
-				if (!listing[i].name.CmpNoCase(file))
-				{
-					if (listing[i].name == file)
-					{
-						entry = listing[i];
-						matchedCase = true;
-						return true;
-					}
-					if (!found)
-					{
-						found = true;
-						entry = listing[i];
-					}
-				}
+				entry = listing[i];
+				matchedCase = true;
+				return true;
 			}
-			if (found)
+			listing.FindFile_CmpNoCase(file);
+			if (i != -1)
 			{
+				entry = listing[i];
 				matchedCase = false;
 				return true;
 			}
-
+			
 			return false;
 		}
 	}
@@ -538,9 +530,19 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 			}
 			if (i != listing.GetCount())
 			{
-				listing[i].name = fileTo;
-				listing[i].unsure = true;
-				listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
+				if (listing[i].dir)
+				{
+					RemoveDir(server, pathFrom, fileFrom);
+					RemoveDir(server, pathFrom, fileTo);
+					UpdateFile(server, pathFrom, fileTo, true, dir);
+				}
+				else
+				{
+					listing[i].name = fileTo;
+					listing[i].unsure = true;
+					listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
+					listing.ClearFindMap();
+				}
 			}
 			return;
 		}
