@@ -8,6 +8,8 @@ CDirectoryListing::CDirectoryListing()
 	m_failed = false;
 	m_referenceCount = 0;
 	m_hasDirs = false;
+	m_searchmap_case = 0;
+	m_searchmap_nocase = 0;
 }
 
 CDirectoryListing::CDirectoryListing(const CDirectoryListing& listing)
@@ -27,6 +29,9 @@ CDirectoryListing::CDirectoryListing(const CDirectoryListing& listing)
 	m_firstListTime = listing.m_firstListTime;
 
 	m_hasDirs = listing.m_hasDirs;
+
+	m_searchmap_case = listing.m_searchmap_case;
+	m_searchmap_nocase = listing.m_searchmap_nocase;
 }
 
 CDirectoryListing::~CDirectoryListing()
@@ -63,12 +68,15 @@ CDirectoryListing& CDirectoryListing::operator=(const CDirectoryListing &a)
 
 	m_hasDirs = a.m_hasDirs;
 
+	m_searchmap_case = a.m_searchmap_case;
+	m_searchmap_nocase = a.m_searchmap_nocase;
+
 	return *this;
 }
 
 wxString CDirentry::dump() const
 {
-	wxString str = wxString::Format(_T("name=%s\nsize=%s\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nhasDate=%d\nhasTime=%d\n"),
+	wxString str = wxString::Format(_T("name=%s\nsize=%s\npermissions=%s\nownerGroup=%s\ndir=%d\nlink=%d\ntarget=%s\nhasDate=%d\nhasTime=%d\nunsure=%d\n"),
 				name.c_str(), size.ToString().c_str(), permissions.c_str(), ownerGroup.c_str(), dir, link,
 				target.c_str(), hasDate, hasTime, unsure);
 
@@ -126,14 +134,23 @@ void CDirectoryListing::SetCount(unsigned int count)
 	if (count == m_entryCount)
 		return;
 
+	const unsigned int old_count = m_entryCount;
+
 	if (!count)
 	{
 		Unref();
 		m_entryCount = 0;
 		return;
 	}
+
+	if (count < old_count)
+	{
+		Copy(false);
+		m_searchmap_case->clear();
+		m_searchmap_nocase->clear();
+	}
 	else
-		Copy();
+		Copy(true);
 
 	wxASSERT(m_pEntries);
 	m_pEntries->resize(count);
@@ -154,7 +171,7 @@ CDirentry& CDirectoryListing::operator[](unsigned int index)
 	// Commented out, too heavy speed penalty
 	// wxASSERT(index < m_entryCount);
 
-	Copy();
+	Copy(true);
 
 	return (*m_pEntries)[index].GetEntry();
 }
@@ -177,6 +194,12 @@ void CDirectoryListing::Unref()
 
 	delete m_referenceCount;
 	m_referenceCount = 0;
+
+	delete m_searchmap_case;
+	m_searchmap_case = 0;
+
+	delete m_searchmap_nocase;
+	m_searchmap_nocase = 0;
 }
 
 void CDirectoryListing::AddRef()
@@ -186,12 +209,16 @@ void CDirectoryListing::AddRef()
 		// New object
 		m_referenceCount = new int(1);
 		m_pEntries = new std::vector<CDirentryObject>;
+
+		m_searchmap_case = new std::multimap<wxString, unsigned int>;
+		m_searchmap_nocase = new std::multimap<wxString, unsigned int>;
+
 		return;
 	}
 	(*m_referenceCount)++;
 }
 
-void CDirectoryListing::Copy()
+void CDirectoryListing::Copy(bool copy_cache)
 {
 	if (!m_referenceCount)
 	{
@@ -208,10 +235,25 @@ void CDirectoryListing::Copy()
 	(*m_referenceCount)--;
 	m_referenceCount = new int(1);
 	
-
 	std::vector<CDirentryObject>* pEntries = new std::vector<CDirentryObject>;
 	*pEntries = *m_pEntries;
 	m_pEntries = pEntries;
+
+	if (copy_cache)
+	{
+		std::multimap<wxString, unsigned int>* searchmap_case = new std::multimap<wxString, unsigned int>;
+		*searchmap_case = *searchmap_case;
+		m_searchmap_case = searchmap_case;
+
+		std::multimap<wxString, unsigned int>* searchmap_nocase = new std::multimap<wxString, unsigned int>;
+		*searchmap_nocase = *searchmap_nocase;
+		m_searchmap_nocase = searchmap_nocase;
+	}
+	else
+	{
+		m_searchmap_case = new std::multimap<wxString, unsigned int>;
+		m_searchmap_nocase = new std::multimap<wxString, unsigned int>;
+	}
 }
 
 void CDirectoryListing::Assign(const std::list<CDirentry> &entries)
@@ -237,7 +279,9 @@ bool CDirectoryListing::RemoveEntry(unsigned int index)
 	if (index >= GetCount())
 		return false;
 
-	Copy();
+	Copy(false);
+	m_searchmap_case->clear();
+	m_searchmap_nocase->clear();
 
 	std::vector<CDirentryObject>::iterator iter = m_pEntries->begin() + index;
 	if (iter->GetEntry().dir)
@@ -347,3 +391,69 @@ void CDirectoryListing::GetFilenames(std::vector<wxString> &names) const
 	for (unsigned int i = 0; i < GetCount(); i++)
 		names.push_back((*this)[i].name);
 }
+
+int CDirectoryListing::FindFile_CmpCase(const wxString& name) const
+{
+	if (!m_pEntries)
+		return -1;
+
+	// Search map
+	std::multimap<wxString, unsigned int>::const_iterator iter = m_searchmap_case->find(name);
+	if (iter != m_searchmap_case->end())
+		return iter->second;
+
+	unsigned int i = m_searchmap_case->size();
+
+	// Build map if not yet complete
+	std::vector<CDirentryObject>::const_iterator entry_iter = m_pEntries->begin() + i;
+	for (; entry_iter != m_pEntries->end(); entry_iter++, i++)
+	{
+		const wxString& entry_name = entry_iter->GetEntry().name;
+		m_searchmap_case->insert(std::pair<wxString, unsigned int>(entry_name, i));
+
+		if (entry_name == name)
+			return i;
+	}
+
+	// Map is complete, item not in it
+	return -1;
+}
+
+int CDirectoryListing::FindFile_CmpNoCase(wxString name) const
+{
+	if (!m_pEntries)
+		return -1;
+
+	name.MakeLower();
+
+	// Search map
+	std::multimap<wxString, unsigned int>::const_iterator iter = m_searchmap_nocase->find(name);
+	if (iter != m_searchmap_nocase->end())
+		return iter->second;
+
+	unsigned int i = m_searchmap_nocase->size();
+
+	// Build map if not yet complete
+	std::vector<CDirentryObject>::const_iterator entry_iter = m_pEntries->begin() + i;
+	for (; entry_iter != m_pEntries->end(); entry_iter++, i++)
+	{
+		wxString entry_name = entry_iter->GetEntry().name;
+		entry_name.MakeLower();
+		m_searchmap_nocase->insert(std::pair<wxString, unsigned int>(entry_name, i));
+
+		if (entry_name == name)
+			return i;
+	}
+
+	// Map is complete, item not in it
+	return -1;
+}
+
+void CDirectoryListing::ClearFindMap()
+{
+	if (!m_searchmap_case)
+		return;
+
+	m_searchmap_case->clear();
+	m_searchmap_nocase->clear();
+}
