@@ -1,5 +1,4 @@
 #include "FileZilla.h"
-#include "asynchostresolver.h"
 #include "ControlSocket.h"
 #include "ftpcontrolsocket.h"
 #include "sftpcontrolsocket.h"
@@ -49,7 +48,6 @@ wxEvent *wxFzEngineEvent::Clone() const
 
 BEGIN_EVENT_TABLE(CFileZillaEnginePrivate, wxEvtHandler)
 	EVT_FZ_ENGINE_NOTIFICATION(wxID_ANY, CFileZillaEnginePrivate::OnEngineEvent)
-	EVT_FZ_ASYNCHOSTRESOLVE(wxID_ANY, CFileZillaEnginePrivate::OnAsyncHostResolver)
 	EVT_TIMER(wxID_ANY, CFileZillaEnginePrivate::OnTimer)
 END_EVENT_TABLE();
 
@@ -79,14 +77,6 @@ CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 	for (std::list<CNotification *>::iterator iter = m_NotificationList.begin(); iter != m_NotificationList.end(); iter++)
 		delete *iter;
 
-	for (std::list<CAsyncHostResolver *>::iterator iter = m_HostResolverThreads.begin(); iter != m_HostResolverThreads.end(); iter++)
-	{
-		CAsyncHostResolver* pResolver = *iter;
-		pResolver->SetObsolete();
-		pResolver->Wait();
-		delete pResolver;					
-	}
-
 	// Remove ourself from the engine list
 	for (std::list<CFileZillaEnginePrivate*>::iterator iter = m_engineList.begin(); iter != m_engineList.end(); iter++)
 		if (*iter == this)
@@ -99,6 +89,9 @@ CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 
 	if (m_pRateLimiter)
 		m_pRateLimiter->Free();
+
+	if (m_engineList.empty())
+		CSocket::Cleanup(true);
 }
 
 bool CFileZillaEnginePrivate::SendEvent(enum EngineNotificationType eventType, int data /*=0*/)
@@ -126,40 +119,6 @@ void CFileZillaEnginePrivate::OnEngineEvent(wxFzEngineEvent &event)
 	}
 }
 
-void CFileZillaEnginePrivate::OnAsyncHostResolver(fzAsyncHostResolveEvent& event)
-{
-	if (m_HostResolverThreads.empty())
-		return;
-	CAsyncHostResolver *pResolver = m_HostResolverThreads.front();
-	m_HostResolverThreads.pop_front();
-
-	std::list<CAsyncHostResolver *> remaining;
-	for (std::list<CAsyncHostResolver *>::iterator iter = m_HostResolverThreads.begin(); iter != m_HostResolverThreads.end(); iter++)
-	{
-		CAsyncHostResolver* pResolver = *iter;
-		pResolver->SetObsolete();
-		if (!pResolver->Done())
-			remaining.push_back(pResolver);
-		else
-		{
-			pResolver->Wait();
-			delete pResolver;					
-		}
-	}
-	m_HostResolverThreads.clear();
-	m_HostResolverThreads = remaining;
-
-	if (!pResolver->Done())
-		m_HostResolverThreads.push_front(pResolver);
-	else
-	{
-		if (!pResolver->Obsolete())
-			m_pControlSocket->ContinueConnect(pResolver->Successful() ? &pResolver->m_Address : 0);
-		pResolver->Wait();
-		delete pResolver;
-	}
-}
-
 bool CFileZillaEnginePrivate::IsBusy() const
 {
 	return m_pCurrentCommand != 0;
@@ -265,9 +224,6 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 		}
 	}
 
-	if (!m_HostResolverThreads.empty())
-		m_HostResolverThreads.front()->SetObsolete();
-
 	return nErrorCode;
 }
 
@@ -300,16 +256,6 @@ unsigned int CFileZillaEnginePrivate::GetNextAsyncRequestNumber()
 	return ++m_asyncRequestCounter;
 }
 
-void CFileZillaEnginePrivate::AddNewAsyncHostResolver(CAsyncHostResolver* pResolver)
-{
-	wxASSERT(pResolver);
-
-	if (!m_HostResolverThreads.empty())
-		m_HostResolverThreads.front()->SetObsolete();
-	
-	m_HostResolverThreads.push_front(pResolver);
-}
-
 // Command handlers
 int CFileZillaEnginePrivate::Connect(const CConnectCommand &command)
 {
@@ -384,6 +330,9 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 	if (!IsConnected())
 		return FZ_REPLY_NOTCONNECTED;
 
+	if (command.GetPath().IsEmpty() && command.GetSubDir() != _T(""))
+		return FZ_REPLY_SYNTAXERROR;
+
 	bool refresh = command.Refresh();
 	if (!command.Refresh() && !command.GetPath().IsEmpty())
 	{
@@ -529,7 +478,6 @@ int CFileZillaEnginePrivate::Chmod(const CChmodCommand& command)
 void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath& path, bool onList, bool modified, bool failed)
 {
 	wxASSERT(m_pControlSocket);
-	wxASSERT(onList || modified);
 	
 	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
 	wxASSERT(pOwnServer);
