@@ -1,6 +1,7 @@
 #include "FileZilla.h"
 #include "ControlSocket.h"
 #include "httpcontrolsocket.h"
+#include <errno.h>
 
 // Connect is special for HTTP: It is done on a per-command basis, so we need
 // to establish a connection before each command.
@@ -88,13 +89,11 @@ CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate *pEngine)
 	: CRealControlSocket(pEngine)
 {
 	m_pRecvBuffer = 0;
-	m_pAddress = 0;
 }
 
 CHttpControlSocket::~CHttpControlSocket()
 {
 	delete [] m_pRecvBuffer;
-	delete m_pAddress;
 }
 
 int CHttpControlSocket::SendNextCommand()
@@ -126,9 +125,10 @@ int CHttpControlSocket::SendNextCommand()
 	return FZ_REPLY_ERROR;
 }
 
-int CHttpControlSocket::ContinueConnect(const wxIPV4address *address)
+
+int CHttpControlSocket::ContinueConnect()
 {
-	LogMessage(__TFILE__, __LINE__, this, Debug_Verbose, _T("CHttpControlSocket::ContinueConnect(%p) m_pEngine=%p"), address, m_pEngine);
+	LogMessage(__TFILE__, __LINE__, this, Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() m_pEngine=%p"), m_pEngine);
 	if (GetCurrentCommandId() != cmd_connect ||
 		!m_pCurrentServer)
 	{
@@ -136,30 +136,6 @@ int CHttpControlSocket::ContinueConnect(const wxIPV4address *address)
 		return DoClose(FZ_REPLY_INTERNALERROR);
 	}
 
-	if (!address)
-	{
-		if (m_pCurrentServer)
-		{
-			delete m_pCurrentServer;
-			m_pCurrentServer = 0;
-		}
-
-		LogMessage(::Error, _("Invalid hostname or host not found"));
-		return ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_CRITICALERROR);
-	}
-
-	if (m_pCurOpData && m_pCurOpData->opId == cmd_connect)
-	{
-		CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
-		pData->host = address->IPAddress();
-		return DoInternalConnect();
-	}
-
-	if (m_pAddress)
-		delete m_pAddress;
-	m_pAddress = new wxIPV4address(*address);
-
-	// Now we got the server's IP address for later use, e.g. transfer commands
 	ResetOperation(FZ_REPLY_OK);
 	return FZ_REPLY_OK;
 }
@@ -186,78 +162,14 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 				return false;
 			}
 
-			CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
-
 			CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
 			switch (pFileExistsNotification->overwriteAction)
 			{
-			case CFileExistsNotification::overwrite:
-				SendNextCommand();
-				break;
-			case CFileExistsNotification::overwriteNewer:
-				if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
-					SendNextCommand();
-				else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
-					SendNextCommand();
-				else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
-					SendNextCommand();
-				else
-				{
-					if (pData->download)
-					{
-						wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-						LogMessage(Status, _("Skipping download of %s"), filename.c_str());
-					}
-					else
-					{
-						LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
-					}
-					ResetOperation(FZ_REPLY_OK);
-				}
-				break;
 			case CFileExistsNotification::resume:
 				ResetOperation(FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED);
 				break;
-			case CFileExistsNotification::rename:
-				if (pData->download)
-				{
-					wxFileName fn = pData->localFile;
-					fn.SetFullName(pFileExistsNotification->newName);
-					pData->localFile = fn.GetFullPath();
-
-					wxStructStat buf;
-					int result;
-					result = wxStat(pData->localFile, &buf);
-					if (!result)
-						pData->localFileSize = buf.st_size;
-					else
-						pData->localFileSize = -1;
-
-					if (CheckOverwriteFile() == FZ_REPLY_OK)
-						SendNextCommand();
-				}
-				else
-				{
-					ResetOperation(FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED);
-					break;
-				}
-				break;
-			case CFileExistsNotification::skip:
-				if (pData->download)
-				{
-					wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-					LogMessage(Status, _("Skipping download of %s"), filename.c_str());
-				}
-				else
-				{
-					LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
-				}
-				ResetOperation(FZ_REPLY_OK);
-				break;
 			default:
-				LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown file exists action: %d"), pFileExistsNotification->overwriteAction);
-				ResetOperation(FZ_REPLY_INTERNALERROR);
-				return false;
+				return SetFileExistsAction(pFileExistsNotification);
 			}
 		}
 		break;
@@ -278,43 +190,72 @@ void CHttpControlSocket::OnReceive()
 		m_recvBufferPos = 0;
 	}
 
-	unsigned int len = m_recvBufferLen - m_recvBufferPos;
-	Read(m_pRecvBuffer + m_recvBufferPos, len);
-	if (Error())
+	int id = GetId();
+
+	do
 	{
-		if (LastError() != wxSOCKET_WOULDBLOCK)
-			ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
-		return;
-	}
+		if (GetState() != connected && GetState() != closing)
+			return;
 
-	int read;
-	if (!(read = LastCount()))
-	{
-		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
-		return;
-	}
+		unsigned int len = m_recvBufferLen - m_recvBufferPos;
+		int error;
+		int read = Read(m_pRecvBuffer + m_recvBufferPos, len, error);
+		if (read == -1)
+		{
+			if (error != EAGAIN)
+			{
+				ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
+			}
+			return;
+		}
 
-	m_pEngine->SetActive(true);
+		m_pEngine->SetActive(true);
 
-	if (!m_pCurOpData || m_pCurOpData->opId == cmd_connect)
-	{
-		// Just ignore all further data
-		m_recvBufferPos = 0;
-		return;
-	}
+		if (!m_pCurOpData || m_pCurOpData->opId == cmd_connect)
+		{
+			// Just ignore all further data
+			m_recvBufferPos = 0;
+			return;
+		}
 
-	m_recvBufferPos += read;
+		m_recvBufferPos += read;
 
-	if (!m_pHttpOpData->m_gotHeader)
-		ParseHeader(m_pHttpOpData);
-	else if (m_pHttpOpData->m_transferEncoding == CHttpOpData::chunked)
-		OnChunkedData(m_pHttpOpData);
-	else
-	{
-		m_pHttpOpData->m_receivedData += m_recvBufferPos;
-		ProcessData(m_pRecvBuffer, m_recvBufferPos);
-		m_recvBufferPos = 0;
+		if (!m_pHttpOpData->m_gotHeader)
+		{
+			if (!read)
+			{
+				ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
+				return;
+			}
+
+			ParseHeader(m_pHttpOpData);
+		}
+		else if (m_pHttpOpData->m_transferEncoding == CHttpOpData::chunked)
+		{
+			if (!read)
+			{
+				ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
+				return;
+			}
+			OnChunkedData(m_pHttpOpData);
+		}
+		else
+		{
+			if (!read)
+			{
+				wxASSERT(!m_recvBufferPos);
+				ProcessData(0, 0);
+				return;
+			}
+			else
+			{
+				m_pHttpOpData->m_receivedData += m_recvBufferPos;
+				ProcessData(m_pRecvBuffer, m_recvBufferPos);
+				m_recvBufferPos = 0;
+			}
+		}
 	}
+	while (GetId() == id);
 }
 
 void CHttpControlSocket::OnConnect()
@@ -384,7 +325,7 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	else
 		pData->opState = filetransfer_transfer;
 
-	int res = InternalConnect(m_pAddress->IPAddress(), m_pCurrentServer->GetPort());
+	int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort());
 	if (res != FZ_REPLY_OK)
 		return res;
 
@@ -402,8 +343,6 @@ int CHttpControlSocket::FileTransferSubcommandResult(int prevResult)
 		return FZ_REPLY_ERROR;
 	}
 
-	//CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
-
 	if (prevResult != FZ_REPLY_OK)
 	{
 		ResetOperation(prevResult);
@@ -442,7 +381,7 @@ int CHttpControlSocket::FileTransferSend()
 			return FZ_REPLY_ERROR;
 		}
 
-		int res = InternalConnect(m_pAddress->IPAddress(), m_pCurrentServer->GetPort());
+		int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort());
 		if (res != FZ_REPLY_OK)
 			return res;
 	}
@@ -451,8 +390,8 @@ int CHttpControlSocket::FileTransferSend()
 	wxString hostWithPort;
 	if (pData->m_newLocation == _T(""))
 	{
-		location = _T("http://") + m_pCurrentServer->GetHost() + pData->remotePath.FormatFilename(pData->remoteFile).c_str();
-		hostWithPort = wxString::Format(_T("%s:%d"), m_pCurrentServer->GetHost().c_str(), m_pCurrentServer->GetPort());
+		location = _T("http://") + m_pCurrentServer->FormatHost() + pData->remotePath.FormatFilename(pData->remoteFile).c_str();
+		hostWithPort = wxString::Format(_T("%s:%d"), m_pCurrentServer->FormatHost(true).c_str(), m_pCurrentServer->GetPort());
 	}
 	else
 	{
@@ -472,7 +411,7 @@ int CHttpControlSocket::FileTransferSend()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CHttpControlSocket::InternalConnect(const wxString& host, unsigned short port)
+int CHttpControlSocket::InternalConnect(wxString host, unsigned short port)
 {
 	LogMessage(Debug_Verbose, _T("CHttpControlSocket::InternalConnect()"));
 
@@ -482,15 +421,7 @@ int CHttpControlSocket::InternalConnect(const wxString& host, unsigned short por
 	pData->port = port;
 
 	if (!IsIpAddress(host))
-	{
-		LogMessage(Status, _("Resolving IP-Address for %s"), host.c_str());
-		CAsyncHostResolver *resolver = new CAsyncHostResolver(m_pEngine, ConvertDomainName(host));
-		m_pEngine->AddNewAsyncHostResolver(resolver);
-
-		resolver->Create();
-		resolver->Run();
-		return FZ_REPLY_WOULDBLOCK;
-	}
+		LogMessage(Status, _("Resolving address of %s"), host.c_str());
 
 	pData->host = host;
 	return DoInternalConnect();
@@ -508,23 +439,19 @@ int CHttpControlSocket::DoInternalConnect()
 	}
 
 	CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
-	LogMessage(Status, _("Connecting to %s:%d..."), m_pAddress->IPAddress().c_str(), pData->port);
+	LogMessage(Status, _("Connecting to %s:%d..."), pData->host.c_str(), pData->port);
 
 	if (m_pBackend)
 		delete m_pBackend;
 	m_pBackend = new CSocketBackend(this, this);
 
-	wxIPV4address addr;
-	addr.Hostname(pData->host);
-	addr.Service(pData->port);
-
-	bool res = wxSocketClient::Connect(addr, false);
-
-	if (res)
+	int res = CSocket::Connect(pData->host, pData->port);
+	if (!res)
 		return FZ_REPLY_OK;
-	else if (LastError() != wxSOCKET_WOULDBLOCK)
-		return ResetOperation(FZ_REPLY_ERROR);
 
+	if (res && res != EINPROGRESS)
+		return ResetOperation(FZ_REPLY_ERROR);
+	
 	return FZ_REPLY_WOULDBLOCK;
 }
 
@@ -592,7 +519,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			{
 				if (m_pRecvBuffer[i + 1] != '\n')
 				{
-					LogMessage(::Error, _("Malformed reply, server not sending proper lineendings"));
+					LogMessage(::Error, _("Malformed reply, server not sending proper line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
@@ -701,7 +628,10 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
-					pData->m_newHostWithPort = wxString::Format(_T("%s:%d"), host.c_str(), port);
+					pData->m_newHostWithPort = wxString::Format(_T("%s:%d"), host.c_str(), (int)port);
+
+					// International domain names
+					host = ConvertDomainName(host);
 
 					return InternalConnect(host, port);
 				}
@@ -803,7 +733,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			{
 				if (p[i + 1] != '\n')
 				{
-					LogMessage(::Error, _("Malformed chunk data: %s"), _("Wrong lineendings"));
+					LogMessage(::Error, _("Malformed chunk data: %s"), _("Wrong line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
@@ -920,11 +850,18 @@ int CHttpControlSocket::ResetOperation(int nErrorCode)
 	return CControlSocket::ResetOperation(nErrorCode);
 }
 
-void CHttpControlSocket::OnClose()
+void CHttpControlSocket::OnClose(int error)
 {
-	char tmp[1];
-	for (Peek(tmp, 1); !Error() && LastCount(); Peek(tmp, 1))
-		OnReceive();
+	LogMessage(Debug_Verbose, _T("CRealControlSocket::OnClose(%d)"), error);
+
+	if (error)
+	{
+		LogMessage(::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error).c_str());
+		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
+		return;
+	}
+	
+	OnReceive();
 
 	// HTTP socket isn't connected outside operations
 	if (!m_pCurOpData)
