@@ -9,8 +9,8 @@ public:
 	virtual ~CHttpControlSocket();
 
 protected:
-	virtual int ContinueConnect(const wxIPV4address *address);
-	virtual bool Connected() const { return m_pCurrentServer != 0; }
+	virtual int ContinueConnect();
+	virtual bool Connected() { return m_pCurrentServer != 0; }
 
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification);
 	virtual int SendNextCommand();
@@ -23,11 +23,11 @@ protected:
 	virtual int FileTransferParseResponse(char* p, unsigned int len);
 	virtual int FileTransferSubcommandResult(int prevResult);
 
-	int InternalConnect(const wxString& host, unsigned short port);
+	int InternalConnect(wxString host, unsigned short port);
 	int DoInternalConnect();
 
 	virtual void OnConnect();
-	virtual void OnClose();
+	virtual void OnClose(int error);
 	virtual void OnReceive();
 
 	virtual int ResetOperation(int nErrorCode);
@@ -39,9 +39,6 @@ protected:
 
 	int ProcessData(char* p, int len);
 
-	// IP address of server
-	wxIPV4address* m_pAddress;
-
 	char* m_pRecvBuffer;
 	unsigned int m_recvBufferPos;
 	static const unsigned int m_recvBufferLen = 4096;
