@@ -221,12 +221,12 @@ wxString CServerPath::GetSafePath() const
 		return _T("");
 
 	wxString safepath;
-	safepath.Printf(_T("%d %d "), m_type, m_prefix.Length());
+	safepath.Printf(_T("%d %d "), m_type, (int)m_prefix.Length());
 	if (m_prefix != _T(""))
 		safepath += m_prefix + _T(" ");
 
 	for (tConstSegmentIter iter = m_Segments.begin(); iter != m_Segments.end(); iter++)
-		safepath += wxString::Format(_T("%d %s "), iter->Length(), iter->c_str());
+		safepath += wxString::Format(_T("%d %s "), (int)iter->Length(), iter->c_str());
 
 	if (!m_Segments.empty())
 		safepath.RemoveLast();
@@ -257,7 +257,7 @@ bool CServerPath::SetSafePath(wxString path)
 			return false;
 		else
 		{
-			// Is root folder, like / on unix like systems.
+			// Is root directory, like / on unix like systems.
 			m_bEmpty = false;
 			return true;
 		}
@@ -443,7 +443,15 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		break;
 	case DOS:
 		{
-			if (dir.Length() >= 2 && dir.c_str()[1] == ':')
+			bool is_relative = false;
+			int sep = dir.Find(traits[m_type].separator);
+			if (sep == -1)
+				sep = dir.Len();
+			int colon = dir.Find(':');
+			if (colon > 0 && colon == sep - 1)
+				is_relative = true;
+
+			if (is_relative)
 				m_Segments.clear();
 			else if (dir[0] == traits[m_type].separator)
 			{
