@@ -7,6 +7,8 @@
 #include "pathcache.h"
 #include "servercapabilities.h"
 #include <wx/tokenzr.h>
+#include "local_filesys.h"
+#include "proxy.h"
 
 class CSftpFileTransferOpData : public CFileTransferOpData
 {
@@ -311,6 +313,7 @@ CSftpControlSocket::~CSftpControlSocket()
 enum connectStates
 {
 	connect_init,
+	connect_proxy,
 	connect_keys,
 	connect_open
 };
@@ -340,7 +343,7 @@ public:
 
 int CSftpControlSocket::Connect(const CServer &server)
 {
-	LogMessage(Status, _("Connecting to %s:%d..."), server.GetHost().c_str(), server.GetPort());
+	LogMessage(Status, _("Connecting to %s..."), server.FormatHost().c_str());
 	SetWait(true);
 
 	m_sftpEncryptionDetails = CSftpEncryptionNotification();
@@ -434,6 +437,14 @@ int CSftpControlSocket::ConnectParseResponse(bool successful, const wxString& re
 	switch (pData->opState)
 	{
 	case connect_init:
+		if (m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_TYPE) && !m_pCurrentServer->GetBypassProxy())
+			pData->opState = connect_proxy;
+		else if (pData->pKeyFiles)
+			pData->opState = connect_keys;
+		else
+			pData->opState = connect_open;
+		break;
+	case connect_proxy:
 		if (pData->pKeyFiles)
 			pData->opState = connect_keys;
 		else
@@ -478,6 +489,35 @@ int CSftpControlSocket::ConnectSend()
 	bool res;
 	switch (pData->opState)
 	{
+	case connect_proxy:
+		{
+			int type;
+			switch (m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_TYPE))
+			{
+			case CProxySocket::HTTP:
+				type = 1;
+				break;
+			case CProxySocket::SOCKS5:
+				type = 2;
+				break;
+			default:
+				LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unsupported proxy type"));
+				DoClose(FZ_REPLY_INTERNALERROR);
+				return FZ_REPLY_ERROR;
+			}
+
+			wxString cmd = wxString::Format(_T("proxy %d \"%s\" %d"), type,
+											m_pEngine->GetOptions()->GetOption(OPTION_PROXY_HOST).c_str(),
+											m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_PORT));
+			wxString user = m_pEngine->GetOptions()->GetOption(OPTION_PROXY_USER);
+			if (user != _T(""))
+				cmd += _T(" \"") + user + _T("\"");
+			wxString pass = m_pEngine->GetOptions()->GetOption(OPTION_PROXY_PASS);
+			if (pass != _T(""))
+				cmd += _T(" \"") + pass + _T("\"");
+			res = Send(cmd);
+		}
+		break;
 	case connect_keys:
 		res = Send(_T("keyfile \"") + pData->pKeyFiles->GetNextToken() + _T("\""));
 		break;
@@ -758,121 +798,9 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	{
 	case reqId_fileexists:
 		{
-			if (!m_pCurOpData || m_pCurOpData->opId != cmd_transfer)
-			{
-				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pNotification->GetRequestID());
-				return false;
-			}
-
-			CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
-
 			CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
-			switch (pFileExistsNotification->overwriteAction)
-			{
-			case CFileExistsNotification::overwrite:
-				SendNextCommand();
-				break;
-			case CFileExistsNotification::overwriteNewer:
-				if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
-					SendNextCommand();
-				else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
-					SendNextCommand();
-				else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
-					SendNextCommand();
-				else
-				{
-					if (pData->download)
-					{
-						wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-						LogMessage(Status, _("Skipping download of %s"), filename.c_str());
-					}
-					else
-					{
-						LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
-					}
-					ResetOperation(FZ_REPLY_OK);
-				}
-				break;
-			case CFileExistsNotification::resume:
-				if (pData->download && pFileExistsNotification->localSize != -1)
-					pData->resume = true;
-				else if (!pData->download && pFileExistsNotification->remoteSize != -1)
-					pData->resume = true;
-				SendNextCommand();
-				break;
-			case CFileExistsNotification::rename:
-				if (pData->download)
-				{
-					wxFileName fn = pData->localFile;
-					fn.SetFullName(pFileExistsNotification->newName);
-					pData->localFile = fn.GetFullPath();
-
-					wxStructStat buf;
-					int result;
-					result = wxStat(pData->localFile, &buf);
-					if (!result)
-						pData->localFileSize = buf.st_size;
-					else
-						pData->localFileSize = -1;
-
-					if (CheckOverwriteFile() == FZ_REPLY_OK)
-						SendNextCommand();
-				}
-				else
-				{
-					pData->remoteFile = pFileExistsNotification->newName;
-
-					CDirectoryListing listing;
-					CDirectoryCache cache;
-					bool is_outdated = false;
-					bool found = cache.Lookup(listing, *m_pCurrentServer, pData->tryAbsolutePath ? pData->remotePath : m_CurrentPath, true, is_outdated);
-					if (found)
-					{
-						bool differentCase = false;
-						bool found = false;
-						for (unsigned int i = 0; i < listing.GetCount(); i++)
-						{
-							if (!listing[i].name.CmpNoCase(pData->remoteFile))
-							{
-								if (listing[i].name != pData->remoteFile)
-									differentCase = true;
-								else
-								{
-									wxLongLong size = listing[i].size;
-									pData->remoteFileSize = size.GetLo() + ((wxFileOffset)size.GetHi() << 32);
-									found = true;
-									break;
-								}
-							}
-						}
-						if (found)
-						{
-							if (CheckOverwriteFile() != FZ_REPLY_OK)
-								break;
-						}
-					}
-					SendNextCommand();
-				}
-				break;
-			case CFileExistsNotification::skip:
-				if (pData->download)
-				{
-					wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-					LogMessage(Status, _("Skipping download of %s"), filename.c_str());
-				}
-				else
-				{
-					LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
-				}
-				ResetOperation(FZ_REPLY_OK);
-				break;
-			default:
-				LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown file exists action: %d"), pFileExistsNotification->overwriteAction);
-				ResetOperation(FZ_REPLY_INTERNALERROR);
-				return false;
-			}
+			return SetFileExistsAction(pFileExistsNotification);
 		}
-		break;
 	case reqId_hostkey:
 	case reqId_hostkeyChanged:
 		{
@@ -1198,6 +1126,9 @@ int CSftpControlSocket::ListSubcommandResult(int prevResult)
 		return FZ_REPLY_ERROR;
 	}
 
+	if (pData->path.IsEmpty())
+		pData->path = m_CurrentPath;
+
 	if (!pData->refresh)
 	{
 		wxASSERT(!pData->pNextOpData);
@@ -1634,17 +1565,16 @@ int CSftpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	pData->download = download;
 	pData->transferSettings = transferSettings;
 
+	wxLongLong size;
+	bool isLink;
+	if (CLocalFileSystem::GetFileInfo(pData->localFile, isLink, &size, 0, 0) == CLocalFileSystem::file)
+		pData->localFileSize = size.GetValue();
+
 	pData->opState = filetransfer_waitcwd;
 
 	if (pData->remotePath.GetType() == DEFAULT)
 		pData->remotePath.SetType(m_pCurrentServer->GetType());
 
-	wxStructStat buf;
-	int result;
-	result = wxStat(pData->localFile, &buf);
-	if (!result)
-		pData->localFileSize = buf.st_size;
-
 	int res = ChangeDir(pData->remotePath);
 	if (res != FZ_REPLY_OK)
 		return res;
@@ -2292,7 +2222,7 @@ int CSftpControlSocket::DeleteSend()
 	wxString filename = pData->path.FormatFilename(file);
 	if (filename == _T(""))
 	{
-		LogMessage(::Error, _T("Filename cannot be constructed for folder %s and filename %s"), pData->path.GetPath().c_str(), file.c_str());
+		LogMessage(::Error, _("Filename cannot be constructed for directory %s and filename %s"), pData->path.GetPath().c_str(), file.c_str());
 		return FZ_REPLY_ERROR;
 	}
 
@@ -2334,7 +2264,7 @@ int CSftpControlSocket::RemoveDir(const CServerPath& path /*=CServerPath()*/, co
 
 	if (!fullPath.AddSegment(subDir))
 	{
-		LogMessage(::Error, _T("Path cannot be constructed for folder %s and subdir %s"), path.GetPath().c_str(), subDir.c_str());
+		LogMessage(::Error, _("Path cannot be constructed for directory %s and subdir %s"), path.GetPath().c_str(), subDir.c_str());
 		return FZ_REPLY_ERROR;
 	}
 
@@ -2753,6 +2683,9 @@ int CSftpControlSocket::ListCheckTimezoneDetection()
 			if (!pData->directoryListing[i].hasTime)
 				continue;
 
+			if (pData->directoryListing[i].link)
+				continue;
+
 			pData->opState = list_mtime;
 			pData->mtime_index = i;
 			return SendNextCommand();
