@@ -7,10 +7,10 @@
 #include <errno.h>
 
 BEGIN_EVENT_TABLE(CTlsSocket, wxEvtHandler)
-EVT_SOCKET(wxID_ANY, CTlsSocket::OnSocketEvent)
+EVT_FZ_SOCKET(wxID_ANY, CTlsSocket::OnSocketEvent)
 END_EVENT_TABLE()
 
-CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, wxSocketBase* pSocket, CControlSocket* pOwner)
+CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
 	: CBackend(pEvtHandler), m_pOwner(pOwner)
 {
 	m_pSocket = pSocket;
@@ -19,12 +19,12 @@ CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, wxSocketBase* pSocket, CContro
 	m_session = 0;
 	m_initialized = false;
 	m_certCredentials = 0;
-	
+
 	m_canReadFromSocket = true;
 	m_canWriteToSocket = true;
 	m_canCheckCloseSocket = false;
 
-	m_canTriggerRead = true;
+	m_canTriggerRead = false;
 	m_canTriggerWrite = true;
 
 	m_tlsState = noconn;
@@ -33,6 +33,7 @@ CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, wxSocketBase* pSocket, CContro
 	m_lastCount = 0;
 	m_lastSuccessful = false;
 
+	m_lastReadFailed = true;
 	m_lastWriteFailed = false;
 	m_writeSkip = 0;
 
@@ -45,6 +46,8 @@ CTlsSocket::CTlsSocket(wxEvtHandler* pEvtHandler, wxSocketBase* pSocket, CContro
 	m_implicitTrustedCert.size = 0;
 
 	m_shutdown_requested = false;
+
+	m_socket_eof = false;
 }
 
 CTlsSocket::~CTlsSocket()
@@ -89,7 +92,7 @@ bool CTlsSocket::Init()
 	}
 
 	gnutls_dh_set_prime_bits(m_session, 512);
-	
+
 	// Set which type of certificates we accept
 	const int cert_type_priority[] = { GNUTLS_CRT_X509, 0 };
 	gnutls_certificate_type_set_priority(m_session, cert_type_priority);
@@ -211,7 +214,7 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 	if (m_pSocketBackend->Error())
 	{
 		const int error = m_pSocketBackend->LastError();
-		if (error == wxSOCKET_WOULDBLOCK)
+		if (error == EAGAIN)
 		{
 			m_canWriteToSocket = false;
 			gnutls_transport_set_errno(m_session, EAGAIN);
@@ -242,39 +245,37 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 		return -1;
 	}
 
-	m_canReadFromSocket = false;
-
 	m_pSocketBackend->Read(data, len);
 	if (m_pSocketBackend->Error())
 	{
-		if (m_pSocketBackend->LastError() == wxSOCKET_WOULDBLOCK)
+		if (m_pSocketBackend->LastError() == EAGAIN)
 		{
+			m_canReadFromSocket = false;
 			if (m_canCheckCloseSocket && !m_pSocketBackend->IsWaiting(CRateLimiter::inbound))
 			{
-				wxSocketEvent evt(m_pSocketBackend->GetId());
-				evt.m_event = wxSOCKET_LOST;
+				CSocketEvent evt(m_pSocketBackend->GetId(), CSocketEvent::close);
 				wxPostEvent(this, evt);
 			}
-
-			gnutls_transport_set_errno(m_session, EAGAIN);
-			return -1;
 		}
 
-		gnutls_transport_set_errno(m_session, 0);
+		gnutls_transport_set_errno(m_session, m_pSocketBackend->LastError());
 		return -1;
 	}
 
 	if (m_canCheckCloseSocket)
 	{
-		wxSocketEvent evt(m_pSocketBackend->GetId());
-		evt.m_event = wxSOCKET_LOST;
+		CSocketEvent evt(m_pSocketBackend->GetId(), CSocketEvent::close);
 		wxPostEvent(this, evt);
 	}
 
-	return m_pSocketBackend->LastCount();
+	unsigned int read = m_pSocketBackend->LastCount();
+	if (!read)
+		m_socket_eof = true;
+
+	return read;
 }
 
-void CTlsSocket::OnSocketEvent(wxSocketEvent& event)
+void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 {
 	wxASSERT(m_pSocket);
 	if (!m_session)
@@ -283,15 +284,15 @@ void CTlsSocket::OnSocketEvent(wxSocketEvent& event)
 	if (event.GetId() != m_pSocketBackend->GetId())
 		return;
 
-	switch (event.GetSocketEvent())
+	switch (event.GetType())
 	{
-	case wxSOCKET_INPUT:
+	case CSocketEvent::read:
 		OnRead();
 		break;
-	case wxSOCKET_OUTPUT:
+	case CSocketEvent::write:
 		OnSend();
 		break;
-	case wxSOCKET_LOST:
+	case CSocketEvent::close:
 		{
 			m_canCheckCloseSocket = true;
 			char tmp[100];
@@ -301,20 +302,22 @@ void CTlsSocket::OnSocketEvent(wxSocketEvent& event)
 				int lastCount = m_pSocketBackend->LastCount();
 
 				if (lastCount)
-					m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing wxSOCKET_LOST"));
+					m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
 				else
+				{
+					m_socket_eof = true;
 					m_socketClosed = true;
+				}
 				OnRead();
 
 				if (lastCount)
 					return;
 			}
 
-			m_pOwner->LogMessage(Debug_Info, _T("CTlsSocket::OnSocketEvent(): wxSOCKET_LOST received"));
+			m_pOwner->LogMessage(Debug_Info, _T("CTlsSocket::OnSocketEvent(): close event received"));
 
 			//Uninit();
-			wxSocketEvent evt(GetId());
-			evt.m_event = wxSOCKET_LOST;
+			CSocketEvent evt(GetId(), CSocketEvent::close);
 			wxPostEvent(m_pEvtHandler, evt);
 		}
 		break;
@@ -332,16 +335,17 @@ void CTlsSocket::OnRead()
 	if (!m_session)
 		return;
 
-	if (gnutls_record_get_direction(m_session))
+	const int direction = gnutls_record_get_direction(m_session);
+	if (direction && !m_lastReadFailed)
 		return;
-		
+
 	if (m_tlsState == handshake)
 		Handshake();
 	if (m_tlsState == closing)
 		ContinueShutdown();
 	else if (m_tlsState == conn)
 	{
-		CheckResumeFailedWrite();
+		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
 }
@@ -355,7 +359,8 @@ void CTlsSocket::OnSend()
 	if (!m_session)
 		return;
 
-	if (!gnutls_record_get_direction(m_session))
+	const int direction = gnutls_record_get_direction(m_session);
+	if (!direction && !m_lastWriteFailed)
 		return;
 
 	if (m_tlsState == handshake)
@@ -364,12 +369,51 @@ void CTlsSocket::OnSend()
 		ContinueShutdown();
 	else if (m_tlsState == conn)
 	{
-		CheckResumeFailedWrite();
+		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
 }
 
-int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/)
+bool CTlsSocket::CopySessionData(const CTlsSocket* pPrimarySocket)
+{
+	size_t session_data_size = 0;
+
+	// Get buffer size
+	int res = gnutls_session_get_data(pPrimarySocket->m_session, 0, &session_data_size);
+	if (res)
+	{
+		m_pOwner->LogMessage(Debug_Info, _T("gnutls_session_get_data on primary socket failed: %d"), res);
+		return false;
+	}
+
+	// Get session data
+	char *session_data = new char[session_data_size];
+	res = gnutls_session_get_data(pPrimarySocket->m_session, session_data, &session_data_size);
+	if (res)
+	{
+		delete [] session_data;
+		m_pOwner->LogMessage(Debug_Info, _T("gnutls_session_get_data on primary socket failed: %d"), res);
+		return false;
+	}
+
+	// Set session data
+	res = gnutls_session_set_data(m_session, session_data, session_data_size);
+	delete [] session_data;
+	if (res)
+	{
+		m_pOwner->LogMessage(Debug_Info, _T("gnutls_session_set_data failed: %d"), res);
+		return false;
+	}
+
+	return true;
+}
+
+bool CTlsSocket::ResumedSession() const
+{
+	return gnutls_session_is_resumed(m_session) != 0;
+}
+
+int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resume /*=false*/)
 {
 	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::Handshake()"));
 	wxASSERT(m_session);
@@ -387,6 +431,9 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/)
 			memcpy(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size);
 			m_implicitTrustedCert.size = cert_list[0].size;
 		}
+
+		if (try_resume)
+			CopySessionData(pPrimarySocket);
 	}
 
 	int res = gnutls_handshake(m_session);
@@ -394,6 +441,9 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/)
 	{
 		m_pOwner->LogMessage(Debug_Info, _T("Handshake successful"));
 
+		if (ResumedSession())
+			m_pOwner->LogMessage(Debug_Info, _T("Session resumed"));
+
 		const wxString& cipherName = GetCipherName();
 		const wxString& macName = GetMacName();
 
@@ -403,20 +453,12 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/)
 		if (res != FZ_REPLY_OK)
 			return res;
 
-		m_tlsState = conn;
-
-		wxSocketEvent evt(GetId());
-		evt.m_event = wxSOCKET_CONNECTION;
-		wxPostEvent(m_pEvtHandler, evt);
-		TriggerEvents();
-
 		if (m_shutdown_requested)
 		{
 			Shutdown();
-			if (!Error() || LastError() != wxSOCKET_WOULDBLOCK)
+			if (!Error() || LastError() != EAGAIN)
 			{
-				wxSocketEvent evt(GetId());
-				evt.m_event = wxSOCKET_LOST;
+				CSocketEvent evt(GetId(), CSocketEvent::close);
 				wxPostEvent(m_pEvtHandler, evt);
 			}
 		}
@@ -426,33 +468,38 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/)
 	else if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED)
 		return FZ_REPLY_WOULDBLOCK;
 
-	Failure(res);
+	Failure(res, 0);
 
 	return FZ_REPLY_ERROR;
 }
 
 void CTlsSocket::Read(void *buffer, unsigned int len)
 {
-	if (m_tlsState == handshake)
+	if (m_tlsState == handshake || m_tlsState == verifycert)
 	{
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		m_lastSuccessful = false;
+		m_lastReadFailed = true;
 		return;
 	}
 	else if (m_tlsState != conn)
 	{
-		m_lastError = wxSOCKET_IOERR;
+		m_lastError = ENOTCONN;
+		m_lastSuccessful = false;
+		return;
+	}
+	else if (m_lastReadFailed)
+	{
+		m_lastError = EAGAIN;
 		m_lastSuccessful = false;
 		return;
 	}
-
-	m_canTriggerRead = true;
 
 	if (m_peekDataLen)
 	{
 		unsigned int min = wxMin(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
-		
+
 		if (min == m_peekDataLen)
 		{
 			m_peekDataLen = 0;
@@ -471,7 +518,7 @@ void CTlsSocket::Read(void *buffer, unsigned int len)
 		m_lastCount = min;
 		return;
 	}
-	
+
 	int res = gnutls_record_recv(m_session, buffer, len);
 	if (res >= 0)
 	{
@@ -480,16 +527,28 @@ void CTlsSocket::Read(void *buffer, unsigned int len)
 
 		if (res > 0)
 			TriggerEvents();
+		else
+		{
+			// Peer did already initiate a shutdown, reply to it
+			gnutls_bye(m_session, GNUTLS_SHUT_WR);
+			// Note: Theoretically this could return a write error.
+			// But we ignore it, since it is perfectly valid for peer
+			// to close the connection after sending its shutdown
+			// notification.
+		}
 
 		return;
 	}
 
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
-		m_lastError = wxSOCKET_WOULDBLOCK;
+	{
+		m_lastError = EAGAIN;
+		m_lastReadFailed = true;
+	}
 	else
 	{
-		Failure(res);
-		m_lastError = wxSOCKET_IOERR;
+		Failure(res, 0);
+		m_lastError = ECONNABORTED;
 	}
 
 	m_lastSuccessful = false;
@@ -498,22 +557,22 @@ void CTlsSocket::Read(void *buffer, unsigned int len)
 
 void CTlsSocket::Write(const void *buffer, unsigned int len)
 {
-	if (m_tlsState == handshake)
+	if (m_tlsState == handshake || m_tlsState == verifycert)
 	{
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		m_lastSuccessful = false;
 		return;
 	}
 	else if (m_tlsState != conn)
 	{
-		m_lastError = wxSOCKET_IOERR;
+		m_lastError = ENOTCONN;
 		m_lastSuccessful = false;
 		return;
 	}
 
 	if (m_lastWriteFailed)
 	{
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		m_lastSuccessful = false;
 		return;
 	}
@@ -551,15 +610,15 @@ void CTlsSocket::Write(const void *buffer, unsigned int len)
 		else
 		{
 			m_lastSuccessful = false;
-			m_lastError = wxSOCKET_WOULDBLOCK;
+			m_lastError = EAGAIN;
 			m_lastWriteFailed = true;
 		}
 	}
 	else
 	{
 		m_lastSuccessful = false;
-		Failure(res);
-		m_lastError = wxSOCKET_IOERR;
+		Failure(res, 0);
+		m_lastError = ECONNABORTED;
 	}
 }
 
@@ -570,32 +629,30 @@ void CTlsSocket::TriggerEvents()
 
 	if (m_canTriggerRead)
 	{
-		wxSocketEvent evt(GetId());
-		evt.m_event = wxSOCKET_INPUT;
+		CSocketEvent evt(GetId(), CSocketEvent::read);
 		wxPostEvent(m_pEvtHandler, evt);
 		m_canTriggerRead = false;
 	}
 
 	if (m_canTriggerWrite)
 	{
-		wxSocketEvent evt(GetId());
-		evt.m_event = wxSOCKET_OUTPUT;
+		CSocketEvent evt(GetId(), CSocketEvent::write);
 		wxPostEvent(m_pEvtHandler, evt);
 		m_canTriggerWrite = false;
 	}
 }
 
-void CTlsSocket::CheckResumeFailedWrite()
+void CTlsSocket::CheckResumeFailedReadWrite()
 {
 	if (m_lastWriteFailed)
 	{
 		int res = gnutls_record_send(m_session, 0, 0);
 		if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
 			return;
-		
+
 		if (res < 0)
 		{
-			Failure(res);
+			Failure(res, ECONNABORTED);
 			return;
 		}
 
@@ -605,17 +662,53 @@ void CTlsSocket::CheckResumeFailedWrite()
 
 		wxASSERT(GNUTLS_E_INVALID_REQUEST == gnutls_record_send(m_session, 0, 0));
 	}
+	if (m_lastReadFailed)
+	{
+		wxASSERT(!m_peekData);
+
+		m_peekDataLen = 65536;
+		m_peekData = new char[m_peekDataLen];
+
+		int res = gnutls_record_recv(m_session, m_peekData, m_peekDataLen);
+		if (res < 0)
+		{
+			m_peekDataLen = 0;
+			delete [] m_peekData;
+			m_peekData = 0;
+			if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN)
+				Failure(res, ECONNABORTED);
+			return;
+		}
+
+		if (!res)
+		{
+			m_peekDataLen = 0;
+			delete [] m_peekData;
+			m_peekData = 0;
+		}
+		else
+			m_peekDataLen = res;
+
+		m_lastReadFailed = false;
+		m_canTriggerRead = true;
+	}
 }
 
-void CTlsSocket::Failure(int code)
+void CTlsSocket::Failure(int code, int socket_error)
 {
 	if (code)
+	{
 		LogError(code);
+		if (code == GNUTLS_E_UNEXPECTED_PACKET_LENGTH && m_socket_eof)
+			m_pOwner->LogMessage(Status, _("Server did not properly shut down TLS connection"));
+	}
 	Uninit();
 
-	wxSocketEvent evt(GetId());
-	evt.m_event = wxSOCKET_LOST;
-	wxPostEvent(m_pEvtHandler, evt);
+	if (socket_error)
+	{
+		CSocketEvent evt(GetId(), CSocketEvent::close, socket_error);
+		wxPostEvent(m_pEvtHandler, evt);
+	}
 }
 
 void CTlsSocket::Peek(void *buffer, unsigned int len)
@@ -624,7 +717,7 @@ void CTlsSocket::Peek(void *buffer, unsigned int len)
 	{
 		int min = wxMin(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
-		
+
 		m_lastCount = min;
 		m_lastSuccessful = true;
 		return;
@@ -652,7 +745,7 @@ void CTlsSocket::Shutdown()
 	if (m_tlsState == closing)
 	{
 		m_lastSuccessful = false;
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		return;
 	}
 
@@ -662,14 +755,14 @@ void CTlsSocket::Shutdown()
 		m_pOwner->LogMessage(Debug_Verbose, _T("Shutdown during handshake, postponing"));
 		m_shutdown_requested = true;
 		m_lastSuccessful = false;
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 		return;
 	}
 
 	if (m_tlsState != conn)
 	{
 		m_lastSuccessful = false;
-		m_lastError = wxSOCKET_IOERR;
+		m_lastError = ECONNABORTED;
 		return;
 	}
 
@@ -685,11 +778,11 @@ void CTlsSocket::Shutdown()
 
 	m_lastSuccessful = false;
 	if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN)
-		m_lastError = wxSOCKET_WOULDBLOCK;
+		m_lastError = EAGAIN;
 	else
 	{
-		Failure(res);
-		m_lastError = wxSOCKET_IOERR;
+		Failure(res, 0);
+		m_lastError = ECONNABORTED;
 	}
 }
 
@@ -702,15 +795,14 @@ void CTlsSocket::ContinueShutdown()
 	{
 		m_tlsState = closed;
 
-		wxSocketEvent evt(GetId());
-		evt.m_event = wxSOCKET_LOST;
+		CSocketEvent evt(GetId(), CSocketEvent::close);
 		wxPostEvent(m_pEvtHandler, evt);
 
 		return;
 	}
 
 	if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN)
-		Failure(res);
+		Failure(res, ECONNABORTED);
 }
 
 void CTlsSocket::TrustCurrentCert(bool trusted)
@@ -725,15 +817,23 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 	{
 		m_tlsState = conn;
 
-		wxSocketEvent evt(GetId());
-		evt.m_event = wxSOCKET_CONNECTION;
-		wxPostEvent(m_pEvtHandler, evt);
+		if (m_lastWriteFailed)
+			m_lastWriteFailed = false;
+		CheckResumeFailedReadWrite();
+
+		if (m_tlsState == conn)
+		{
+			CSocketEvent evt(GetId(), CSocketEvent::connection);
+			wxPostEvent(m_pEvtHandler, evt);
+		}
+
 		TriggerEvents();
+
 		return;
 	}
 
 	m_pOwner->LogMessage(::Error, _("Remote certificate not trusted."));
-	Failure(0);
+	Failure(0, ECONNABORTED);
 }
 
 static wxString bin2hex(const unsigned char* in, size_t size)
@@ -762,7 +862,7 @@ int CTlsSocket::VerifyCertificate()
 	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509)
 	{
 		m_pOwner->LogMessage(::Debug_Warning, _T("Unsupported certificate type"));
-		Failure(0);
+		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
@@ -771,7 +871,7 @@ int CTlsSocket::VerifyCertificate()
 	if (!cert_list || !cert_list_size)
 	{
 		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_certificate_get_peers returned no certificates"));
-		Failure(0);
+		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
@@ -781,11 +881,14 @@ int CTlsSocket::VerifyCertificate()
 			memcmp(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size))
 		{
 			m_pOwner->LogMessage(::Error, _("Primary connection and data connection certificates don't match."));
-			Failure(0);
+			Failure(0, ECONNABORTED);
 			return FZ_REPLY_ERROR;
 		}
-		
+
 		TrustCurrentCert(true);
+
+		if (m_tlsState != conn)
+			return FZ_REPLY_ERROR;
 		return FZ_REPLY_OK;
 	}
 
@@ -793,14 +896,14 @@ int CTlsSocket::VerifyCertificate()
 	if (gnutls_x509_crt_init(&cert))
 	{
 		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_crt_init failed"));
-		Failure(0);
+		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (gnutls_x509_crt_import(cert, cert_list, GNUTLS_X509_FMT_DER))
 	{
 		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_crt_import failed"));
-		Failure(0);
+		Failure(0, ECONNABORTED);
 		gnutls_x509_crt_deinit(cert);
 		return FZ_REPLY_ERROR;
 	}
@@ -824,7 +927,7 @@ int CTlsSocket::VerifyCertificate()
 		algoName = wxString(pAlgo, wxConvUTF8);
 
 	//int version = gnutls_x509_crt_get_version(cert);
-	
+
 	wxString subject, issuer;
 
 	size = 0;
@@ -847,7 +950,7 @@ int CTlsSocket::VerifyCertificate()
 	if (subject == _T(""))
 	{
 		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_get_dn failed"));
-		Failure(0);
+		Failure(0, ECONNABORTED);
 		gnutls_x509_crt_deinit(cert);
 		return FZ_REPLY_ERROR;
 	}
@@ -872,7 +975,7 @@ int CTlsSocket::VerifyCertificate()
 	if (issuer == _T(""))
 	{
 		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_get_issuer_dn failed"));
-		Failure(0);
+		Failure(0, ECONNABORTED);
 		gnutls_x509_crt_deinit(cert);
 		return FZ_REPLY_ERROR;
 	}
