@@ -1,3 +1,9 @@
+#include <wx/defs.h>
+#ifdef __WXMSW__
+// For AF_INET6
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
 #include "FileZilla.h"
 #include "transfersocket.h"
 #include "ftpcontrolsocket.h"
@@ -5,9 +11,15 @@
 #include "optionsbase.h"
 #include "iothread.h"
 #include "tlssocket.h"
+#include <errno.h>
+#ifndef __WXMSW__
+#include <sys/socket.h>
+#endif
+#include "proxy.h"
+#include "servercapabilities.h"
 
 BEGIN_EVENT_TABLE(CTransferSocket, wxEvtHandler)
-	EVT_SOCKET(wxID_ANY, CTransferSocket::OnSocketEvent)
+	EVT_FZ_SOCKET(wxID_ANY, CTransferSocket::OnSocketEvent)
 	EVT_IOTHREAD(wxID_ANY, CTransferSocket::OnIOThreadEvent)
 END_EVENT_TABLE();
 
@@ -19,6 +31,7 @@ CTransferSocket::CTransferSocket(CFileZillaEnginePrivate *pEngine, CFtpControlSo
 	m_pSocketServer = 0;
 	m_pSocket = 0;
 	m_pBackend = 0;
+	m_pProxyBackend = 0;
 	m_pTlsSocket = 0;
 
 	m_pDirectoryListingParser = 0;
@@ -49,6 +62,7 @@ CTransferSocket::~CTransferSocket()
 {
 	if (m_transferEndReason == none)
 		m_transferEndReason = successful;
+	delete m_pProxyBackend;
 	if (m_pTlsSocket)
 		delete m_pTlsSocket;
 	else if (m_pBackend)
@@ -81,66 +95,157 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	delete m_pSocketServer;
 	m_pSocketServer = 0;
 
-	wxIPV4address addr;
-	addr.AnyAddress();
-	addr.Service(0);
-
 	m_pSocketServer = CreateSocketServer();
 	if (!m_pSocketServer)
-		return _T("");
-
-	if (!m_pSocketServer->GetLocal(addr))
 	{
-		delete m_pSocketServer;
-		m_pSocketServer = 0;
+		m_pControlSocket->LogMessage(::Debug_Warning, _T("CreateSocketServer failed"));
 		return _T("");
 	}
 
-	wxString portArguments = ip;
-	portArguments += wxString::Format(_T(",%d,%d"), addr.Service() / 256, addr.Service() % 256);
-	portArguments.Replace(_T("."), _T(","));
+	int error;
+	int port = m_pSocketServer->GetLocalPort(error);
+	if (port == -1)
+	{
+		delete m_pSocketServer;
+		m_pSocketServer = 0;
 
-	m_pSocketServer->SetEventHandler(*this);
-	m_pSocketServer->SetNotify(wxSOCKET_CONNECTION_FLAG);
-	m_pSocketServer->Notify(true);
+		m_pControlSocket->LogMessage(::Debug_Warning, _T("GetLocalPort failed: %s"), CSocket::GetErrorDescription(error).c_str());
+		return _T("");
+	}
+
+	wxString portArguments;
+	if (m_pSocketServer->GetAddressFamily() == AF_INET6)
+	{
+		portArguments = wxString::Format(_T("|2|%s|%d|"), ip.c_str(), port);
+	}
+	else
+	{
+		portArguments = ip;
+		portArguments += wxString::Format(_T(",%d,%d"), port / 256, port % 256);
+		portArguments.Replace(_T("."), _T(","));
+	}
 
 	return portArguments;
 }
 
-void CTransferSocket::OnSocketEvent(wxSocketEvent &event)
+void CTransferSocket::OnSocketEvent(CSocketEvent &event)
 {
-	switch (event.GetSocketEvent())
+	if (m_pProxyBackend && event.GetId() == m_pProxyBackend->GetId())
 	{
-	case wxSOCKET_CONNECTION:
-		OnConnect(event);
+		switch (event.GetType())
+		{
+		case CSocketEvent::connection:
+			{
+				const int error = event.GetError();
+				if (error)
+				{
+					m_pControlSocket->LogMessage(::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error).c_str());
+					TransferEnd(failure);
+				}
+				else
+				{
+					delete m_pProxyBackend;
+					m_pProxyBackend = 0;
+					OnConnect();
+				}
+			}
+			return;
+		case CSocketEvent::close:
+			{
+				const int error = event.GetError();
+				m_pControlSocket->LogMessage(::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error).c_str());
+				TransferEnd(failure);
+			}
+			return;
+		default:
+			// Uninteresting
+			return;
+		}
+		return;
+	}
+
+	if (m_pSocketServer && event.GetId() == m_pSocketServer->GetId())
+	{
+		if (event.GetType() == CSocketEvent::connection)
+			OnAccept(event.GetError());
+		else
+			m_pControlSocket->LogMessage(::Debug_Info, _T("Unhandled socket event %d from listening socket"), event.GetType());
+		return;
+	}
+
+	if (m_pBackend)
+	{
+		if (event.GetId() != m_pBackend->GetId())
+		{
+			m_pControlSocket->LogMessage(::Debug_Info, _T("Skipping socket event %d, id mismatch."), event.GetType());
+			return;
+		}
+	}
+	else
+	{
+		if (!m_pSocket || m_pSocket->GetId() != event.GetId())
+		{
+			m_pControlSocket->LogMessage(m_pSocket ? ::Debug_Debug : ::Debug_Info, _T("Skipping socket event %d, no socket or id mismatch."), event.GetType());
+			return;
+		}
+	}
+
+	switch (event.GetType())
+	{
+	case CSocketEvent::connection:
+		OnConnect();
 		break;
-	case wxSOCKET_INPUT:
+	case CSocketEvent::read:
 		OnReceive();
 		break;
-	case wxSOCKET_OUTPUT:
+	case CSocketEvent::write:
 		OnSend();
 		break;
-	case wxSOCKET_LOST:
-		OnClose(event);
+	case CSocketEvent::close:
+		OnClose(event.GetError());
+		break;
+	case CSocketEvent::hostaddress:
+		// Booooring. No seriously, we connect by IP already, nothing to resolve
+		break;
+	default:
+		m_pControlSocket->LogMessage(::Debug_Info, _T("Unhandled socket event %d"), event.GetType());
 		break;
 	}
 }
 
-void CTransferSocket::OnConnect(wxSocketEvent &event)
+void CTransferSocket::OnAccept(int error)
+{
+	m_pControlSocket->SetAlive();
+	m_pControlSocket->LogMessage(::Debug_Verbose, _T("CTransferSocket::OnAccept(%d)"), error);
+
+	if (!m_pSocketServer)
+	{
+		m_pControlSocket->LogMessage(::Debug_Warning, _T("No socket server in OnAccept"), error);
+		return;
+	}
+
+	m_pSocket = m_pSocketServer->Accept(error);
+	if (!m_pSocket)
+	{
+		if (error == EAGAIN)
+			m_pControlSocket->LogMessage(::Debug_Verbose, _T("No pending connection"));
+		else
+		{
+			m_pControlSocket->LogMessage(::Status, _("Could not accept connection: %s"), CSocket::GetErrorDescription(error).c_str());
+			TransferEnd(transfer_failure);
+		}
+		return;
+	}
+	delete m_pSocketServer;
+	m_pSocketServer = 0;
+
+	OnConnect();
+}
+
+void CTransferSocket::OnConnect()
 {
 	m_pControlSocket->SetAlive();
 	m_pControlSocket->LogMessage(::Debug_Verbose, _T("CTransferSocket::OnConnect"));
-	if (m_pSocketServer)
-	{
-		m_pSocket = m_pSocketServer->Accept(false);
-		if (!m_pSocket)
-		{
-			TransferEnd(transfer_failure);
-			return;
-		}
-		delete m_pSocketServer;
-		m_pSocketServer = 0;
-	}
 
 	if (!m_pSocket)
 	{
@@ -156,6 +261,13 @@ void CTransferSocket::OnConnect(wxSocketEvent &event)
 			return;
 		}
 	}
+	else if (m_pTlsSocket)
+	{
+		if (CServerCapabilities::GetCapability(*m_pControlSocket->m_pCurrentServer, tls_resume) == unknown)
+		{
+			CServerCapabilities::SetCapability(*m_pControlSocket->m_pCurrentServer, tls_resume, m_pTlsSocket->ResumedSession() ? yes : no);
+		}
+	}
 
 	if (m_bActive)
 		TriggerPostponedEvents();
@@ -181,137 +293,155 @@ void CTransferSocket::OnReceive()
 
 	if (m_transferMode == list)
 	{
-		char *pBuffer = new char[4096];
-		m_pBackend->Read(pBuffer, 4096);
-		if (m_pBackend->Error())
+		while (true)
 		{
-			delete [] pBuffer;
-			int error = m_pBackend->LastError();
-			if (error == wxSOCKET_NOERROR)
-				TransferEnd(successful);
-			else if (error != wxSOCKET_WOULDBLOCK)
+			char *pBuffer = new char[4096];
+			m_pBackend->Read(pBuffer, 4096);
+			if (m_pBackend->Error())
 			{
-				m_pControlSocket->LogMessage(Debug_Warning, _T("Read failed with error %d"), error);
-				TransferEnd(transfer_failure);
+				delete [] pBuffer;
+				int error = m_pBackend->LastError();
+				if (error != EAGAIN)
+				{
+					m_pControlSocket->LogMessage(::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
+					TransferEnd(transfer_failure);
+				}
+				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
+				{
+					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
+					AddPendingEvent(evt);
+				}
+				return;
 			}
-			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
-			{
-				wxSocketEvent evt(m_pBackend->GetId());
-				evt.m_event = wxSOCKET_LOST;
-				wxPostEvent(this, evt);
-			}
-			return;
-		}
-		int numread = m_pBackend->LastCount();
+			int numread = m_pBackend->LastCount();
 
-		if (numread > 0)
-		{
-			m_pDirectoryListingParser->AddData(pBuffer, numread);
-			m_pEngine->SetActive(true);
-			if (!m_madeProgress)
+			if (numread > 0)
 			{
-				m_madeProgress = 2;
-				m_pControlSocket->SetTransferStatusMadeProgress();
+				m_pDirectoryListingParser->AddData(pBuffer, numread);
+				m_pEngine->SetActive(true);
+				if (!m_madeProgress)
+				{
+					m_madeProgress = 2;
+					m_pControlSocket->SetTransferStatusMadeProgress();
+				}
+				m_pControlSocket->UpdateTransferStatus(numread);
+				if (m_onCloseCalled)
+				{
+					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
+					AddPendingEvent(evt);
+				}
 			}
-			m_pControlSocket->UpdateTransferStatus(numread);
-			if (m_onCloseCalled)
+			else
 			{
-				wxSocketEvent evt(m_pBackend->GetId());
-				evt.m_event = wxSOCKET_LOST;
-				wxPostEvent(this, evt);
-			}
-		}
-		else
-		{
-			delete [] pBuffer;
-			if (!numread)
-				TransferEnd(successful);
-			else if (numread < 0)
-			{
-				m_pControlSocket->LogMessage(Debug_Warning, _T("  numread < 0"));
-				TransferEnd(transfer_failure);
+				delete [] pBuffer;
+				if (!numread)
+					TransferEnd(successful);
+				else if (numread < 0)
+				{
+					m_pControlSocket->LogMessage(Debug_Warning, _T("  numread < 0"));
+					TransferEnd(transfer_failure);
+				}
+				return;
 			}
 		}
 	}
 	else if (m_transferMode == download)
 	{
-		if (!CheckGetNextWriteBuffer())
-			return;
-
-		m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen);
-		if (m_pBackend->Error())
+		while (true)
 		{
-			int error = m_pBackend->LastError();
-			if (error == wxSOCKET_NOERROR)
+			if (!CheckGetNextWriteBuffer())
+				return;
+
+			m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen);
+			if (m_pBackend->Error())
+			{
+				int error = m_pBackend->LastError();
+				if (error != EAGAIN)
+				{
+					m_pControlSocket->LogMessage(::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
+					TransferEnd(transfer_failure);
+				}
+				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
+				{
+					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
+					AddPendingEvent(evt);
+				}
+				return;
+			}
+			int numread = m_pBackend->LastCount();
+
+			if (numread > 0)
+			{
+				m_pEngine->SetActive(true);
+				if (!m_madeProgress)
+				{
+					m_madeProgress = 2;
+					m_pControlSocket->SetTransferStatusMadeProgress();
+				}
+				m_pControlSocket->UpdateTransferStatus(numread);
+
+				m_pTransferBuffer += numread;
+				m_transferBufferLen -= numread;
+
+				if (!CheckGetNextWriteBuffer())
+					return;
+
+				if (m_onCloseCalled && m_transferEndReason == none)
+				{
+					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
+					AddPendingEvent(evt);
+				}
+			}
+			else //!numread
+			{
 				FinalizeWrite();
-			else if (error != wxSOCKET_WOULDBLOCK)
-				TransferEnd(transfer_failure);
-			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
-			{
-				wxSocketEvent evt(m_pBackend->GetId());
-				evt.m_event = wxSOCKET_LOST;
-				wxPostEvent(this, evt);
-			}
-			return;
-		}
-		int numread = m_pBackend->LastCount();
-
-		if (numread > 0)
-		{
-			m_pEngine->SetActive(true);
-			if (!m_madeProgress)
-			{
-				m_madeProgress = 2;
-				m_pControlSocket->SetTransferStatusMadeProgress();
-			}
-			m_pControlSocket->UpdateTransferStatus(numread);
-
-			m_pTransferBuffer += numread;
-			m_transferBufferLen -= numread;
-
-			CheckGetNextWriteBuffer();
-
-			if (m_onCloseCalled && m_transferEndReason == none)
-			{
-				wxSocketEvent evt(m_pBackend->GetId());
-				evt.m_event = wxSOCKET_LOST;
-				wxPostEvent(this, evt);
+				break;
 			}
 		}
-		else //!numread
-			FinalizeWrite();
 	}
 	else if (m_transferMode == resumetest)
 	{
-		char buffer[2];
-		m_pBackend->Read(buffer, 2);
-		if (m_pBackend->Error())
+		while (true)
 		{
-			if (m_pBackend->LastError() != wxSOCKET_WOULDBLOCK)
-				TransferEnd(transfer_failure);
-			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
+			char buffer[2];
+			m_pBackend->Read(buffer, 2);
+			if (m_pBackend->Error())
 			{
-				wxSocketEvent evt(m_pBackend->GetId());
-				evt.m_event = wxSOCKET_LOST;
-				wxPostEvent(this, evt);
+				const int error = m_pBackend->LastError();
+				if (error != EAGAIN)
+				{
+					m_pControlSocket->LogMessage(::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
+					TransferEnd(transfer_failure);
+				}
+				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
+				{
+					CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
+					AddPendingEvent(evt);
+				}
+				return;
 			}
-			return;
-		}
-		int numread = m_pBackend->LastCount();
-		if (!numread)
-		{
-			TransferEnd(successful);
-			return;
-		}
-		m_transferBufferLen += numread;
+			int numread = m_pBackend->LastCount();
+			if (!numread)
+			{
+				if (m_transferBufferLen == 1)
+					TransferEnd(successful);
+				else
+					TransferEnd(failed_resumetest);
+				return;
+			}
+			m_transferBufferLen += numread;
 
-		if (m_transferBufferLen > 1)
-			TransferEnd(failed_resumetest);
-		else if (m_onCloseCalled)
-		{
-			wxSocketEvent evt(m_pBackend->GetId());
-			evt.m_event = wxSOCKET_LOST;
-			wxPostEvent(this, evt);
+			if (m_transferBufferLen > 1)
+			{
+				TransferEnd(failed_resumetest);
+				return;
+			}
+			else if (m_onCloseCalled)
+			{
+				CSocketEvent evt(m_pBackend->GetId(), CSocketEvent::close);
+				AddPendingEvent(evt);
+				return;
+			}
 		}
 	}
 }
@@ -362,26 +492,26 @@ void CTransferSocket::OnSend()
 	if (m_pBackend->Error())
 	{
 		int error = m_pBackend->LastError();
-		if (error == wxSOCKET_WOULDBLOCK)
+		if (error == EAGAIN)
 		{
 			if (!m_madeProgress)
 			{
-				m_pControlSocket->LogMessage(::Debug_Debug, _T("First wxSOCKET_WOULDBLOCK in CTransferSocket::OnSend()"));
+				m_pControlSocket->LogMessage(::Debug_Debug, _T("First EAGAIN in CTransferSocket::OnSend()"));
 				m_madeProgress = 1;
 				m_pControlSocket->SetTransferStatusMadeProgress();
 			}
 		}
-		else if (error != wxSOCKET_NOERROR)
+		else
 		{
-			m_pControlSocket->LogMessage(::Error, _("Error %d writing to socket"), error);
+			m_pControlSocket->LogMessage(Error, _T("Could not write to transfer socket: %s"), CSocket::GetErrorDescription(error).c_str());
 			TransferEnd(transfer_failure);
 		}
 	}
 }
 
-void CTransferSocket::OnClose(wxSocketEvent &event)
+void CTransferSocket::OnClose(int error)
 {
-	m_pControlSocket->LogMessage(::Debug_Verbose, _T("CTransferSocket::OnClose"));
+	m_pControlSocket->LogMessage(::Debug_Verbose, _T("CTransferSocket::OnClose(%d)"), error);
 	m_onCloseCalled = true;
 
 	if (m_transferEndReason != none)
@@ -411,17 +541,36 @@ void CTransferSocket::OnClose(wxSocketEvent &event)
 		return;
 	}
 
-	if (!m_pTlsSocket)
-		m_pSocket->SetNotify(wxSOCKET_OUTPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG);
+	if (error)
+	{
+		m_pControlSocket->LogMessage(::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error).c_str());
+		TransferEnd(transfer_failure);
+		return;
+	}
 
 	char buffer[100];
 	m_pBackend->Peek(&buffer, 100);
 	if (!m_pBackend->Error() && m_pBackend->LastCount())
 	{
+#ifndef __WXMSW__
+		wxFAIL_MSG(_T("Peek isn't supposed to return data after close notification"));
+#endif
+
+		// MSDN says this:
+		//   FD_CLOSE being posted after all data is read from a socket.
+		//   An application should check for remaining data upon receipt
+		//   of FD_CLOSE to avoid any possibility of losing data.
+		// First half is actually plain wrong.
 		OnReceive();
 
 		return;
 	}
+	else if (m_pBackend->Error() && m_pBackend->LastError() != EAGAIN)
+	{
+		m_pControlSocket->LogMessage(::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error).c_str());
+		TransferEnd(transfer_failure);
+		return;
+	}
 
 	if (m_transferMode == resumetest)
 	{
@@ -442,32 +591,39 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 	delete m_pSocketServer;
 	m_pSocketServer = 0;
 
-	wxSocketClient* pSocketClient = new wxSocketClient(wxSOCKET_NOWAIT);
+	m_pSocket = new CSocket(this, 0);
 
-	pSocketClient->SetEventHandler(*this);
-	pSocketClient->SetNotify(wxSOCKET_INPUT_FLAG | wxSOCKET_OUTPUT_FLAG | wxSOCKET_CONNECTION_FLAG | wxSOCKET_LOST_FLAG);
-	pSocketClient->Notify(true);
-
-	wxIPV4address addr;
-	addr.Hostname(host);
-	addr.Service(port);
-
-	bool res = pSocketClient->Connect(addr, false);
-
-	if (!res && pSocketClient->LastError() != wxSOCKET_WOULDBLOCK)
+	if (m_pControlSocket->m_pProxyBackend)
 	{
-		delete pSocketClient;
-		return false;
+		m_pProxyBackend = new CProxySocket(this, m_pSocket, m_pControlSocket);
+
+		int res = m_pProxyBackend->Handshake(m_pControlSocket->m_pProxyBackend->GetProxyType(),
+											 host, port,
+											 m_pControlSocket->m_pProxyBackend->GetUser(), m_pControlSocket->m_pProxyBackend->GetPass());
+
+		if (res != EINPROGRESS)
+		{
+			delete m_pProxyBackend;
+			m_pProxyBackend = 0;
+			delete m_pSocket;
+			m_pSocket = 0;
+			return false;
+		}
+		int error;
+		host = m_pControlSocket->GetPeerIP();
+		port = m_pControlSocket->GetRemotePort(error);
 	}
 
-	SetSocketBufferSizes(pSocketClient);
+	SetSocketBufferSizes(m_pSocket);
 
-	m_pSocket = pSocketClient;
-
-	if (res)
+	int res = m_pSocket->Connect(host, port);
+	if (res && res != EINPROGRESS)
 	{
-		if (!InitBackend())
-			return false;
+		delete m_pProxyBackend;
+		m_pProxyBackend = 0;
+		delete m_pSocket;
+		m_pSocket = 0;
+		return false;
 	}
 
 	return true;
@@ -483,7 +639,7 @@ void CTransferSocket::SetActive()
 	}
 
 	m_bActive = true;
-	if (m_pSocket && m_pSocket->IsConnected())
+	if (m_pSocket && m_pSocket->GetState() == CSocket::connected)
 		TriggerPostponedEvents();
 }
 
@@ -497,6 +653,12 @@ void CTransferSocket::TransferEnd(enum TransferEndReason reason)
 
 	delete m_pSocketServer;
 	m_pSocketServer = 0;
+
+	if (m_pProxyBackend)
+	{
+		delete m_pProxyBackend;
+		m_pProxyBackend = 0;
+	}
 	if (m_pTlsSocket)
 	{
 		if (m_pBackend == m_pTlsSocket)
@@ -515,13 +677,14 @@ void CTransferSocket::TransferEnd(enum TransferEndReason reason)
 	m_pEngine->SendEvent(engineTransferEnd);
 }
 
-wxSocketServer* CTransferSocket::CreateSocketServer(const wxIPV4address& addr)
+CSocket* CTransferSocket::CreateSocketServer(int port)
 {
-	wxSocketServer* pServer = new wxSocketServer(addr, wxSOCKET_NOWAIT | wxSOCKET_REUSEADDR);
-	if (!pServer->Ok())
+	CSocket* pServer = new CSocket(this, CBackend::GetNextId());
+	int res = pServer->Listen(m_pControlSocket->GetAddressFamily(), port);
+	if (res)
 	{
+		m_pControlSocket->LogMessage(::Debug_Verbose, _T("Could not listen on port %d: %s"), port, CSocket::GetErrorDescription(res).c_str());
 		delete pServer;
-		pServer = 0;
 		return 0;
 	}
 
@@ -530,18 +693,13 @@ wxSocketServer* CTransferSocket::CreateSocketServer(const wxIPV4address& addr)
 	return pServer;
 }
 
-wxSocketServer* CTransferSocket::CreateSocketServer()
+CSocket* CTransferSocket::CreateSocketServer()
 {
-	wxIPV4address addr, controladdr;
-	addr.AnyAddress();
-
 	if (!m_pEngine->GetOptions()->GetOptionVal(OPTION_LIMITPORTS))
 	{
 		// Ask the systen for a port
-		addr.Service(0);
-		wxSocketServer* pServer = CreateSocketServer(addr);
-		if (pServer)
-			return pServer;
+		CSocket* pServer = CreateSocketServer(0);
+		return pServer;
 	}
 
 	// Try out all ports in the port range.
@@ -549,7 +707,7 @@ wxSocketServer* CTransferSocket::CreateSocketServer()
 	// increase the port step by step
 
 	// Windows only: I think there's a bug in the socket implementation of
-	// Windows: Even if using wxSOCKET_REUSEADDR, using the same local address
+	// Windows: Even if using SO_REUSEADDR, using the same local address
 	// twice will fail unless there are a couple of minutes between the
 	// connection attempts. This may cause problems if transferring lots of
 	// files with a narrow port range.
@@ -563,22 +721,21 @@ wxSocketServer* CTransferSocket::CreateSocketServer()
 
 	if (start < low || start > high)
 	{
-		srand( (unsigned)time( NULL ) );
-		start = rand() * (high - low) / (RAND_MAX < INT_MAX ? RAND_MAX + 1 : RAND_MAX) + low;
+		start = GetRandomNumber(low, high);
+		wxASSERT(start >= low && start <= high);
 	}
 
-	wxSocketServer* pServer = 0;
+	CSocket* pServer = 0;
 
 	int count = high - low + 1;
 	while (count--)
 	{
-		addr.Service(start++);
-		pServer = CreateSocketServer(addr);
+		pServer = CreateSocketServer(start++);
 		if (pServer)
 			break;
+		if (start > high)
+			start = low;
 	}
-	if (start > high)
-		start = low;
 
 	return pServer;
 }
@@ -631,7 +788,7 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 				m_pTlsSocket->Shutdown();
 				if (m_pTlsSocket->Error())
 				{
-					if (m_pTlsSocket->LastError() != wxSOCKET_WOULDBLOCK)
+					if (m_pTlsSocket->LastError() != EAGAIN)
 						TransferEnd(transfer_failure);
 					return false;
 				}
@@ -688,10 +845,14 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 		return false;
 	}
 
-	if (!m_pTlsSocket->Handshake(pPrimaryTlsSocket))
+	bool try_resume = CServerCapabilities::GetCapability(*m_pControlSocket->m_pCurrentServer, tls_resume) != no;
+
+	int res = m_pTlsSocket->Handshake(pPrimaryTlsSocket, try_resume);
+	if (res && res != FZ_REPLY_WOULDBLOCK)
 	{
 		delete m_pTlsSocket;
 		m_pTlsSocket = 0;
+		return false;
 	}
 
 	m_pBackend = m_pTlsSocket;
@@ -733,13 +894,11 @@ bool CTransferSocket::InitBackend()
 	return true;
 }
 
-void CTransferSocket::SetSocketBufferSizes(wxSocketBase* pSocket)
+void CTransferSocket::SetSocketBufferSizes(CSocket* pSocket)
 {
 	wxCHECK_RET(pSocket, _("SetSocketBufferSize called without socket"));
-	
-	int value = m_pEngine->GetOptions()->GetOptionVal(OPTION_SOCKET_BUFFERSIZE_SEND);
-	pSocket->SetOption(SOL_SOCKET, SO_SNDBUF, &value, sizeof(value));
 
-	value = m_pEngine->GetOptions()->GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
-	pSocket->SetOption(SOL_SOCKET, SO_RCVBUF, &value, sizeof(value));
+	const int size_read = m_pEngine->GetOptions()->GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
+	const int size_write = m_pEngine->GetOptions()->GetOptionVal(OPTION_SOCKET_BUFFERSIZE_SEND);
+	pSocket->SetBufferSizes(size_read, size_write);
 }
