@@ -35,6 +35,9 @@
 #include <wx/xrc/xh_tree.h>
 #include <wx/xrc/xh_hyperlink.h>
 #include "xh_toolb_ex.h"
+#ifdef __WXMSW__
+#include <wx/socket.h>
+#endif
 
 #if defined(__WXMAC__) || defined(__UNIX__)
 #include <wx/stdpaths.h>
@@ -148,10 +151,17 @@ bool CheckForWin2003FirewallBug()
 
 bool CFileZillaApp::OnInit()
 {
+	srand( (unsigned)time( NULL ) );
+
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
 	//wxHandleFatalExceptions();
 #endif
 
+#ifdef __WXMSW__
+	// Need to call WSAStartup. Let wx do that for us
+	wxSocketBase::Initialize();
+#endif
+
 	wxSystemOptions::SetOption(_T("msw.remap"), 0);
 	wxSystemOptions::SetOption(_T("mac.listctrl.always_use_generic"), 1);
 
@@ -182,7 +192,8 @@ bool CFileZillaApp::OnInit()
 			wxString error;
 
 			wxLocale *loc = wxGetLocale();
-			if (!loc)
+			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
+			if (!loc || !currentInfo)
 			{
 				if (!pInfo)
 					error.Printf(_("Failed to set language to %s, using default system language."),
@@ -193,12 +204,8 @@ bool CFileZillaApp::OnInit()
 			}
 			else
 			{
-				const wxLanguageInfo* currentInfo = loc->GetLanguageInfo(loc->GetLanguage());
-				wxString currentName;
-				if (currentInfo)
-					currentName = currentInfo->CanonicalName;
+				wxString currentName = currentInfo->CanonicalName;
 
-				if (currentInfo)
 				if (!pInfo)
 					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
 						language.c_str(), loc->GetLocale(),
@@ -284,13 +291,23 @@ FileZilla will timeout on big transfers.\
 	m_pWrapEngine = new CWrapEngine();
 	m_pWrapEngine->LoadCache();
 
-	wxFrame *frame = new CMainFrame();
+	CMainFrame *frame = new CMainFrame();
 	frame->Show(true);
 	SetTopWindow(frame);
 
+	frame->ProcessCommandLine();
+
 	return true;
 }
 
+#ifdef __WXMSW__
+int CFileZillaApp::OnExit()
+{
+	wxSocketBase::Shutdown();
+	return wxApp::OnExit();
+}
+#endif
+
 bool CFileZillaApp::FileExists(const wxString& file) const
 {
 	int pos = file.Find('*');
