@@ -293,19 +293,7 @@ bool CLocalListView::DisplayDir(wxString dirname)
 		if (IsComparing())
 			ExitComparisonMode();
 
-		// Clear selection
-		int item = -1;
-		while (true)
-		{
-			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-			if (item == -1)
-				break;
-			SetSelection(item, false);
-
-		}
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-		if (item != -1)
-			SetItemState(item, 0, wxLIST_STATE_FOCUSED);
+		ClearSelection();
 		focused = _T("..");
 
 		if (GetItemCount())
@@ -335,6 +323,9 @@ bool CLocalListView::DisplayDir(wxString dirname)
 		data.dir = true;
 		data.icon = -2;
 		data.name = _T("..");
+#ifdef __WXMSW__
+		data.label = _T("..");
+#endif
 		data.size = -1;
 		data.hasTime = 0;
 		m_fileData.push_back(data);
@@ -388,10 +379,13 @@ regular_dir:
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
+#ifdef __WXMSW__
+			data.label = data.name;
+#endif
 			data.hasTime = data.lastModified.IsValid();
 
 			m_fileData.push_back(data);
-			if (!filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
+			if (!filter.FilenameFiltered(data.name, dirname, data.dir, data.size, true, data.attributes))
 			{
 				if (data.dir)
 					totalDirCount++;
@@ -638,6 +632,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	wxFileName fn(m_dir, data->name);
 
 	m_pQueue->QueueFile(false, false, fn.GetFullPath(), data->name, path, *pServer, data->size);
+	m_pQueue->QueueFile_Finish(true);
 }
 
 #ifdef __WXMSW__
@@ -696,6 +691,18 @@ void CLocalListView::DisplayDrives()
 		CLocalFileData data;
 		data.flags = normal;
 		data.name = path;
+
+		// Get the label of the drive
+		wxChar* pVolumeName = new wxChar[501];
+		int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
+		BOOL res = GetVolumeInformation(pDrive, pVolumeName, 500, 0, 0, 0, 0, 0);
+		SetErrorMode(oldErrorMode);
+		if (res && pVolumeName[0])
+			data.label = data.name + _T(" (") + pVolumeName + _T(")");
+		else
+			data.label = data.name;
+		delete [] pVolumeName;
+
 		data.dir = true;
 		data.icon = -2;
 		data.size = -1;
@@ -748,6 +755,9 @@ void CLocalListView::DisplayShares(wxString computer)
 			CLocalFileData data;
 			data.flags = normal;
 			data.name = p->shi1_netname;
+#ifdef __WXMSW__
+			data.label = data.name;
+#endif
 			data.dir = true;
 			data.icon = -2;
 			data.size = -1;
@@ -1103,7 +1113,6 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_DELETE"), false);
 		pMenu->Enable(XRCID("ID_RENAME"), false);
-		pMenu->Enable(XRCID("ID_OPEN"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 	}
 	else if (count > 1)
@@ -1113,10 +1122,9 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 	}
 	else if (selectedDir)
-	{
-		pMenu->Enable(XRCID("ID_OPEN"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
-	}
+	if (fillCount)
+		pMenu->Enable(XRCID("ID_OPEN"), false);
 
 	PopupMenu(pMenu);
 	delete pMenu;
@@ -1135,6 +1143,17 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 			return;
 	}
 
+	const CServer* pServer = m_pState->GetServer();
+	if (!pServer)
+	{
+		wxBell();
+		return;
+	}
+
+	bool added = false;
+
+	bool queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
+
 	item = -1;
 	while (true)
 	{
@@ -1142,25 +1161,18 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		if (item == -1)
 			break;
 
-		CLocalFileData *data = GetData(item);
+		const CLocalFileData *data = GetData(item);
 		if (!data)
-			return;
+			break;
 
 		if (data->flags == fill)
 			continue;
 
-		const CServer* pServer = m_pState->GetServer();
-		if (!pServer)
-		{
-			wxBell();
-			return;
-		}
-
 		CServerPath path = m_pState->GetRemotePath();
 		if (path.IsEmpty())
 		{
 			wxBell();
-			return;
+			break;
 		}
 
 		if (data->dir)
@@ -1175,9 +1187,12 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		{
 			wxFileName fn(m_dir, data->name);
 
-			m_pQueue->QueueFile(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, fn.GetFullPath(), data->name, path, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, fn.GetFullPath(), data->name, path, *pServer, data->size);
+			added = true;
 		}
 	}
+	if (added)
+		m_pQueue->QueueFile_Finish(!queue_only);
 }
 
 void CLocalListView::OnMenuMkdir(wxCommandEvent& event)
@@ -1315,7 +1330,7 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 	{
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); i++)
 		{
-			CLocalFileData *data = GetData(i);
+			const CLocalFileData *data = GetData(i);
 			if (data && data->flags != fill)
 				SetSelection(i, true);
 			else
@@ -1403,6 +1418,20 @@ bool Rename(wxWindow* parent, wxString dir, wxString from, wxString to)
 
 void CLocalListView::OnEndLabelEdit(wxListEvent& event)
 {
+#ifdef __WXMAC__
+	int item = event.GetIndex();
+	if (item != -1)
+	{
+		int from = item;
+		int to = item;
+		if (from)
+			from--;
+		if (to < GetItemCount() - 1)
+			to++;
+		RefreshItems(from, to);
+	}
+#endif
+
 	if (event.IsEditCancelled())
 		return;
 
@@ -1445,6 +1474,9 @@ void CLocalListView::OnEndLabelEdit(wxListEvent& event)
 	else
 	{
 		data->name = newname;
+#ifdef __WXMSW__
+		data->label = data->name;
+#endif
 		m_pState->RefreshLocal();
 	}
 }
@@ -1480,7 +1512,7 @@ void CLocalListView::ApplyCurrentFilter()
 		const CLocalFileData& data = m_fileData[i];
 		if (data.flags == fill)
 			continue;
-		if (filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
+		if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes))
 		{
 			hidden++;
 			continue;
@@ -1522,24 +1554,30 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 {
 	std::list<wxString> selectedNames;
 	// Remember which items were selected
-	int item = -1;
-	while (true)
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (GetSelectedItemCount())
+#endif
 	{
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
-			break;
-		const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-		if (data.flags != fill)
+		int item = -1;
+		while (true)
 		{
-			if (data.dir)
-				selectedNames.push_back(_T("d") + data.name);
-			else
-				selectedNames.push_back(_T("-") + data.name);
+			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+			if (item == -1)
+				break;
+			const CLocalFileData &data = m_fileData[m_indexMapping[item]];
+			if (data.flags != fill)
+			{
+				if (data.dir)
+					selectedNames.push_back(_T("d") + data.name);
+				else
+					selectedNames.push_back(_T("-") + data.name);
+			}
+			SetSelection(item, false);
 		}
-		SetSelection(item, false);
 	}
 
-	item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
+	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 	if (item != -1)
 	{
 		const CLocalFileData &data = m_fileData[m_indexMapping[item]];
@@ -1696,11 +1734,14 @@ void CLocalListView::RefreshFile(const wxString& file)
 	data.flags = normal;
 	data.icon = -2;
 	data.name = file;
+#ifdef __WXMSW__
+	data.label = file;
+#endif
 	data.dir = type == CLocalFileSystem::dir;
 	data.hasTime = data.lastModified.IsValid();
 
 	CFilterManager filter;
-	if (filter.FilenameFiltered(data.name, data.dir, data.size, true, data.attributes))
+	if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes))
 		return;
 
 	// Look if file data already exists
@@ -1714,24 +1755,30 @@ void CLocalListView::RefreshFile(const wxString& file)
 		// Update file list status bar
 		if (m_pFilelistStatusBar)
 		{
-			int item = -1;
-			while (true)
+#ifndef __WXMSW__
+			// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+			if (GetSelectedItemCount())
+#endif
 			{
-				item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-				if (item == -1)
-					break;
-				if (m_indexMapping[item] != i)
-					continue;
+				int item = -1;
+				while (true)
+				{
+					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+					if (item == -1)
+						break;
+					if (m_indexMapping[item] != i)
+						continue;
 
-				if (oldData.dir)
-					m_pFilelistStatusBar->UnselectDirectory();
-				else
-					m_pFilelistStatusBar->UnselectFile(oldData.size);
-				if (data.dir)
-					m_pFilelistStatusBar->SelectDirectory();
-				else
-					m_pFilelistStatusBar->SelectFile(data.size);
-				break;
+					if (oldData.dir)
+						m_pFilelistStatusBar->UnselectDirectory();
+					else
+						m_pFilelistStatusBar->UnselectFile(oldData.size);
+					if (data.dir)
+						m_pFilelistStatusBar->SelectDirectory();
+					else
+						m_pFilelistStatusBar->SelectFile(data.size);
+					break;
+				}
 			}
 
 			if (oldData.dir)
@@ -1762,6 +1809,11 @@ void CLocalListView::RefreshFile(const wxString& file)
 		return;
 	}
 
+	if (data.dir)
+		m_pFilelistStatusBar->AddDirectory();
+	else
+		m_pFilelistStatusBar->AddFile(data.size);
+
 	wxString focused;
 	std::list<wxString> selectedNames;
 	if (IsComparing())
@@ -1941,7 +1993,13 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 		return _T("");
 
 	if (!column)
+	{
+#ifdef __WXMSW__
+		return data->label;
+#else
 		return data->name;
+#endif
+	}
 	else if (column == 1)
 	{
 		if (data->size < 0)
@@ -2040,7 +2098,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		return;
 	case CEditHandler::edit:
 		{
-			int res = wxMessageBox(_("A file with that name is already being edited. Discard old file and download the new file?"), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
+			int res = wxMessageBox(wxString::Format(_("A file with that name is already being edited. Do you want to reopen '%s'?"), fn.GetFullPath().c_str()), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
 			if (res != wxYES)
 			{
 				wxBell();
@@ -2063,7 +2121,12 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 {
 	long item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	if (item < 1)
+	if (item == -1)
+	{
+		wxLaunchDefaultBrowser(CState::GetAsURL(m_dir));
+		return;
+	}
+	else if (item < (m_hasParent ? 1 : 0))
 	{
 		wxBell();
 		return;
@@ -2090,12 +2153,19 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 		return;
 	}
 
-	if (data->dir || data->flags == fill)
+	if (data->flags == fill)
 	{
 		wxBell();
 		return;
 	}
 
+	if (data->dir)
+	{
+		wxString dir = CState::Canonicalize(m_dir, data->name);
+		wxLaunchDefaultBrowser(CState::GetAsURL(dir));
+		return;
+	}
+
 	wxFileName fn(m_dir, data->name);
 
 	wxString cmd = pEditHandler->GetSystemOpenCommand(fn.GetFullPath());
