@@ -83,6 +83,7 @@ BEGIN_EVENT_TABLE(CMainFrame, wxFrame)
 	EVT_MENU(XRCID("ID_VIEW_QUEUE"), CMainFrame::OnToggleQueueView)
 	EVT_MENU(wxID_ABOUT, CMainFrame::OnMenuHelpAbout)
 	EVT_TOOL(XRCID("ID_TOOLBAR_FILTER"), CMainFrame::OnFilter)
+	EVT_TOOL_RCLICKED(XRCID("ID_TOOLBAR_FILTER"), CMainFrame::OnFilterRightclicked)
 #if FZ_MANUALUPDATECHECK
 	EVT_MENU(XRCID("ID_CHECKFORUPDATES"), CMainFrame::OnCheckForUpdates)
 #endif //FZ_MANUALUPDATECHECK
@@ -217,6 +218,12 @@ CMainFrame::CMainFrame()
 	m_transferStatusTimer.SetOwner(this, TRANSFERSTATUS_TIMER_ID);
 	m_closeEventTimer.SetOwner(this);
 
+	if (CFilterManager::HasActiveFilters(true))
+	{
+		if (COptions::Get()->GetOptionVal(OPTION_FILTERTOGGLESTATE))
+			CFilterManager::ToggleFilters();
+	}
+
 	CreateMenus();
 	CreateToolBar();
 	CreateQuickconnectBar();
@@ -382,7 +389,7 @@ CMainFrame::CMainFrame()
 	SetAcceleratorTable(accel);
 
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	if (COptions::Get()->GetOptionVal(OPTION_UPDATECHECK))
+	if (!COptions::Get()->GetDefaultVal(DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK))
 	{
 		m_pUpdateWizard = new CUpdateWizard(this);
 		m_pUpdateWizard->InitAutoUpdateCheck();
@@ -412,8 +419,6 @@ CMainFrame::CMainFrame()
 
 	InitMenubarState();
 	InitToolbarState();
-
-	ProcessCommandLine();
 }
 
 CMainFrame::~CMainFrame()
@@ -571,15 +576,32 @@ bool CMainFrame::CreateMenus()
 		wxLogError(_("Cannot load main menu from resource file"));
 	}
 
-#if !FZ_MANUALUPDATECHECK
-	if (m_pMenuBar)
+#if FZ_MANUALUPDATECHECK
+	if (COptions::Get()->GetDefaultVal(DEFAULT_DISABLEUPDATECHECK))
+#endif
 	{
-		wxMenu *helpMenu;
-		wxMenuItem* pUpdateItem = m_pMenuBar->FindItem(XRCID("ID_CHECKFORUPDATES"), &helpMenu);
-		if (pUpdateItem)
-			helpMenu->Delete(pUpdateItem);
+		if (m_pMenuBar)
+		{
+			wxMenu *helpMenu;
+
+			wxMenuItem* pUpdateItem = m_pMenuBar->FindItem(XRCID("ID_CHECKFORUPDATES"), &helpMenu);
+			if (pUpdateItem)
+			{
+				// Get rid of separator
+				unsigned int count = helpMenu->GetMenuItemCount();
+				for (unsigned int i = 0; i < count - 1; i++)
+				{
+					if (helpMenu->FindItemByPosition(i) == pUpdateItem)
+					{
+						helpMenu->Delete(helpMenu->FindItemByPosition(i + 1));
+						break;
+					}
+				}
+
+				helpMenu->Delete(pUpdateItem);
+			}
+		}
 	}
-#endif //!FZ_MANUALUPDATECHECK
 
 	if (COptions::Get()->GetOptionVal(OPTION_DEBUG_MENU) && m_pMenuBar)
 	{
@@ -1056,8 +1078,7 @@ bool CMainFrame::CreateToolBar()
 		m_pToolBar->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 #endif
 
-	CFilterManager filters;
-	m_pToolBar->ToggleTool(XRCID("ID_TOOLBAR_FILTER"), filters.HasActiveFilters());
+	m_pToolBar->ToggleTool(XRCID("ID_TOOLBAR_FILTER"), CFilterManager::HasActiveFilters());
 	SetToolBar(m_pToolBar);
 
 	if (m_pQuickconnectBar)
@@ -1356,6 +1377,9 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	m_pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
 	m_pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 
+	bool filters_toggled = CFilterManager::HasActiveFilters(true) && !CFilterManager::HasActiveFilters(false);
+	COptions::Get()->SetOption(OPTION_FILTERTOGGLESTATE, filters_toggled ? 1 : 0);
+
 	Destroy();
 }
 
@@ -1785,7 +1809,7 @@ void CMainFrame::OnCheckForUpdates(wxCommandEvent& event)
 	wxString version(PACKAGE_VERSION, wxConvLocal);
 	if (version[0] < '0' || version[0] > '9')
 	{
-		wxMessageBox(_("Executable contains no version info, cannot check for updates."), _("Updatecheck failed"), wxICON_ERROR, this);
+		wxMessageBox(_("Executable contains no version info, cannot check for updates."), _("Check for updates failed"), wxICON_ERROR, this);
 		return;
 	}
 
@@ -1921,7 +1945,8 @@ void CMainFrame::ConnectToSite(CSiteManagerItemData* const pData)
 {
 	wxASSERT(pData);
 
-	if (pData->m_server.GetLogonType() == ASK)
+	if (pData->m_server.GetLogonType() == ASK ||
+		(pData->m_server.GetLogonType() == INTERACTIVE && pData->m_server.GetUser() == _T("")))
 	{
 		if (!CLoginManager::Get().GetPassword(pData->m_server, false, pData->m_server.GetName()))
 			return;
@@ -1936,7 +1961,7 @@ void CMainFrame::ConnectToSite(CSiteManagerItemData* const pData)
 void CMainFrame::CheckChangedSettings()
 {
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	if (COptions::Get()->GetOptionVal(OPTION_UPDATECHECK))
+	if (!COptions::Get()->GetDefaultVal(DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK))
 	{
 		if (!m_pUpdateWizard)
 		{
@@ -2428,3 +2453,18 @@ void CMainFrame::ProcessCommandLine()
 		m_pState->Connect(server, true, path);
 	}
 }
+
+void CMainFrame::OnFilterRightclicked(wxCommandEvent& event)
+{
+	const bool active = CFilterManager::HasActiveFilters();
+
+	CFilterManager::ToggleFilters();
+
+	if (active == CFilterManager::HasActiveFilters())
+		return;
+
+	if (m_pState)
+		m_pState->NotifyHandlers(STATECHANGE_APPLYFILTER);
+	if (m_pToolBar)
+		m_pToolBar->ToggleTool(XRCID("ID_TOOLBAR_FILTER"), !active);
+}
