@@ -4,7 +4,6 @@
 #include "Options.h"
 #include "StatusView.h"
 #include "statuslinectrl.h"
-#include "../tinyxml/tinyxml.h"
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
 #include "ipcmutex.h"
@@ -24,6 +23,7 @@
 #include <wx/sound.h>
 #include "local_filesys.h"
 #include "statusbar.h"
+#include "recursive_operation.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -189,9 +189,14 @@ public:
 		m_didSendEvent = false;
 		m_threadWaiting = false;
 	}
-	~CFolderProcessingThread() { }
 
-	void GetFiles(std::list<t_newEntry> &entryList)
+	virtual ~CFolderProcessingThread()
+	{
+		for (std::list<t_newEntry*>::iterator iter = m_entryList.begin(); iter != m_entryList.end(); iter++)
+			delete *iter;
+	}
+
+	void GetFiles(std::list<t_newEntry*> &entryList)
 	{
 		wxASSERT(entryList.empty());
 		wxMutexLocker locker(m_sync);
@@ -208,7 +213,7 @@ public:
 
 protected:
 
-	void AddEntry(const t_newEntry& entry)
+	void AddEntry(t_newEntry* entry)
 	{
 		m_sync.Lock();
 		m_entryList.push_back(entry);
@@ -291,8 +296,8 @@ protected:
 						{
 							// Empty directory
 
-							t_newEntry entry;
-							entry.remotePath.SetSafePath(pair.remotePath.GetSafePath().c_str());
+							t_newEntry *entry = new t_newEntry;
+							entry->remotePath.SetSafePath(pair.remotePath.GetSafePath().c_str());
 
 							AddEntry(entry);
 						}
@@ -331,7 +336,7 @@ protected:
 						continue;
 					}
 
-					if (!m_filters.FilenameFiltered(file, true, -1, true, attributes))
+					if (!m_filters.FilenameFiltered(file, currentLocalPath, true, -1, true, attributes))
 					{
 						CFolderScanItem::t_dirPair pair;
 						pair.localPath = fullName;
@@ -342,13 +347,16 @@ protected:
 				}
 				else if (type == CLocalFileSystem::file)
 				{
-					if (!m_filters.FilenameFiltered(file, false, size, true, attributes))
+					if (!m_filters.FilenameFiltered(file, currentLocalPath, false, size, true, attributes))
 					{
-						t_newEntry entry;
-						entry.localFile = fullName.c_str();
-						entry.remoteFile = file.c_str();
-						entry.remotePath.SetSafePath(currentRemotePath.GetSafePath().c_str());
-						entry.size = size;
+						t_newEntry* entry = new t_newEntry;
+
+						{
+							entry->localFile = fullName.c_str();
+							entry->remoteFile = file.c_str();
+							entry->remotePath.SetSafePath(currentRemotePath.GetSafePath().c_str());
+							entry->size = size;
+						}
 
 						AddEntry(entry);
 					}
@@ -382,7 +390,7 @@ protected:
 	}
 
 	// Access has to be guarded by m_sync
-	std::list<t_newEntry> m_entryList;
+	std::list<t_newEntry*> m_entryList;
 
 	CQueueView* m_pOwner;
 	CFolderScanItem* m_pFolderItem;
@@ -475,9 +483,14 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 
 	InsertItem(pServerItem, fileItem);
 
+	return true;
+}
+
+void CQueueView::QueueFile_Finish(const bool start)
+{
 	CommitChanges();
 
-	if (!m_activeMode && !queueOnly)
+	if (!m_activeMode && start)
 	{
 		m_activeMode = 1;
 		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
@@ -489,8 +502,6 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 	UpdateStatusLinePositions();
 
 	Refresh(false);
-
-	return true;
 }
 
 bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, const CRemoteDataObject& dataObject)
@@ -605,7 +616,7 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 				else
 				{
 					// Set default file exists action
-					pFileExistsNotification->overwriteAction = (enum CFileExistsNotification::OverwriteAction)pEngineData->pItem->m_defaultFileExistsAction;
+					pFileExistsNotification->overwriteAction = pEngineData->pItem->m_defaultFileExistsAction;
 				}
 			}
 			break;
@@ -943,7 +954,8 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 		// Increase error count only if item didn't make any progress. This keeps
 		// user interaction at a minimum if connection is unstable.
 
-		if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->m_madeProgress)
+		if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->m_madeProgress &&
+			(replyCode & FZ_REPLY_WRITEFAILED) != FZ_REPLY_WRITEFAILED)
 		{
 			// Don't increase error count if there has been progress
 			CFileItem* pItem = (CFileItem*)pEngineData->pItem;
@@ -958,6 +970,12 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 				pEngineData->pItem->m_statusMessage = _("Timeout");
 			else if (replyCode & FZ_REPLY_DISCONNECTED)
 				pEngineData->pItem->m_statusMessage = _("Disconnected from server");
+			else if ((replyCode & FZ_REPLY_WRITEFAILED) == FZ_REPLY_WRITEFAILED)
+			{
+				pEngineData->pItem->m_statusMessage = _("Could not write to local file");
+				ResetEngine(*pEngineData, failure);
+				return;
+			}
 			else if ((replyCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
 			{
 				pEngineData->pItem->m_statusMessage = _("Could not start transfer");
@@ -1370,6 +1388,14 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
 			(*iter)->QueueImmediateFiles();
 
+		CState* const pState = m_pMainFrame->GetState();
+		if (pState)
+		{
+			CRecursiveOperation* pRecursiveOperation = pState->GetRecursiveOperationHandler();
+			if (pRecursiveOperation && pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download)
+				pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
+		}
+
 		UpdateStatusLinePositions();
 
 		// Send active engines the cancel command
@@ -1395,6 +1421,8 @@ bool CQueueView::SetActive(bool active /*=true*/)
 			}
 		}
 
+		m_pMainFrame->GetState()->NotifyHandlers(STATECHANGE_QUEUEPROCESSING);
+
 		return m_activeCount == 0;
 	}
 	else
@@ -1642,23 +1670,25 @@ void CQueueView::OnFolderThreadComplete(wxCommandEvent& event)
 	ProcessUploadFolderItems();
 }
 
-bool CQueueView::QueueFiles(const std::list<t_newEntry> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const int defaultFileExistsAction)
+bool CQueueView::QueueFiles(const std::list<t_newEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const enum CFileExistsNotification::OverwriteAction defaultFileExistsAction)
 {
 	wxASSERT(pServerItem);
 
-	for (std::list<t_newEntry>::const_iterator iter = entryList.begin(); iter != entryList.end(); iter++)
+	for (std::list<t_newEntry*>::const_iterator iter = entryList.begin(); iter != entryList.end(); iter++)
 	{
-		const t_newEntry& entry = *iter;
+		const t_newEntry* entry = *iter;
 
 		CFileItem* fileItem;
-		if (entry.localFile != _T(""))
+		if (entry->localFile != _T(""))
 		{
-			fileItem = new CFileItem(pServerItem, queueOnly, download, entry.localFile, entry.remoteFile, entry.remotePath, entry.size);
-			fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? entry.remoteFile : wxFileName(entry.localFile).GetFullName(), entry.remotePath.GetType());
+			fileItem = new CFileItem(pServerItem, queueOnly, download, entry->localFile, entry->remoteFile, entry->remotePath, entry->size);
+			fileItem->m_transferSettings.binary = ShouldUseBinaryMode(download ? entry->remoteFile : wxFileName(entry->localFile).GetFullName(), entry->remotePath.GetType());
 			fileItem->m_defaultFileExistsAction = defaultFileExistsAction;
 		}
 		else
-			fileItem = new CFolderItem(pServerItem, queueOnly, entry.remotePath, _T(""));
+			fileItem = new CFolderItem(pServerItem, queueOnly, entry->remotePath, _T(""));
+
+		delete entry;
 
 		InsertItem(pServerItem, fileItem);
 	}
@@ -1699,9 +1729,10 @@ void CQueueView::SaveQueue()
 
 	WriteToFile(pDocument);
 
-	if (!pDocument->GetDocument()->SaveFile(file.GetFullPath().mb_str()))
+	wxString error;
+	if (!SaveXmlFile(file.GetFullPath(), pDocument, &error))
 	{
-		wxString msg = wxString::Format(_("Could not write \"%s\", the queue could not be saved."), file.GetFullPath().c_str());
+		wxString msg = wxString::Format(_("Could not write \"%s\", the queue could not be saved.\n%s"), file.GetFullPath().c_str(), error.c_str());
 		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 
@@ -1940,6 +1971,13 @@ void CQueueView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	const bool has_selection = GetSelectedItemCount() != 0;
+#else
+	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
+#endif
+
 	pMenu->Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
 	pMenu->Check(XRCID("ID_ACTIONAFTER_DISABLE"), IsActionAfter(ActionAfterState_Disabled));
 	pMenu->Check(XRCID("ID_ACTIONAFTER_CLOSE"), IsActionAfter(ActionAfterState_Close));
@@ -1951,12 +1989,10 @@ void CQueueView::OnContextMenu(wxContextMenuEvent& event)
 	pMenu->Check(XRCID("ID_ACTIONAFTER_REBOOT"), IsActionAfter(ActionAfterState_Reboot));
 	pMenu->Check(XRCID("ID_ACTIONAFTER_SHUTDOWN"), IsActionAfter(ActionAfterState_Shutdown));
 #endif
-	pMenu->Enable(XRCID("ID_REMOVE"), GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1);
+	pMenu->Enable(XRCID("ID_REMOVE"), has_selection);
 
-	const bool hasSelection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
-
-	pMenu->Enable(XRCID("ID_PRIORITY"), hasSelection);
-	pMenu->Enable(XRCID("ID_DEFAULT_FILEEXISTSACTION"), hasSelection);
+	pMenu->Enable(XRCID("ID_PRIORITY"), has_selection);
+	pMenu->Enable(XRCID("ID_DEFAULT_FILEEXISTSACTION"), has_selection);
 #ifdef __WXMSW__
 	pMenu->Enable(XRCID("ID_ACTIONAFTER"), m_actionAfterWarnDialog == NULL);
 #endif
@@ -2047,9 +2083,15 @@ void CQueueView::RemoveAll()
 	// for removal
 
 	// First, clear all selections
-	int item;
-	while ((item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-		SetItemState(item, 0, wxLIST_STATE_SELECTED);
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (GetSelectedItemCount())
+#endif
+	{
+		int item;
+		while ((item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+			SetItemState(item, 0, wxLIST_STATE_SELECTED);
+	}
 
 	std::vector<CServerItem*> newServerList;
 	m_itemCount = 0;
@@ -2077,6 +2119,12 @@ void CQueueView::RemoveAll()
 
 void CQueueView::OnRemoveSelected(wxCommandEvent& event)
 {
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (!GetSelectedItemCount())
+		return;
+#endif
+
 	std::list<CQueueItem*> selectedItems;
 	long item = -1;
 	while (true)
@@ -2227,7 +2275,7 @@ void CQueueView::OnFolderThreadFiles(wxCommandEvent& event)
 	wxASSERT(!m_queuedFolders[1].empty());
 	CFolderScanItem* pItem = m_queuedFolders[1].front();
 
-	std::list<t_newEntry> entryList;
+	std::list<t_newEntry*> entryList;
 	m_pFolderProcessingThread->GetFiles(entryList);
 	QueueFiles(entryList, pItem->Queued(), false, (CServerItem*)pItem->GetTopLevelItem(), pItem->m_defaultFileExistsAction);
 
@@ -2236,7 +2284,7 @@ void CQueueView::OnFolderThreadFiles(wxCommandEvent& event)
 	RefreshItem(pItem);
 }
 
-void CQueueView::SetDefaultFileExistsAction(int action, const enum TransferDirection direction)
+void CQueueView::SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction)
 {
 	for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
 		(*iter)->SetDefaultFileExistsAction(action, direction);
@@ -2244,14 +2292,20 @@ void CQueueView::SetDefaultFileExistsAction(int action, const enum TransferDirec
 
 void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &event)
 {
-	if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	const bool has_selection = GetSelectedItemCount() != 0;
+#else
+	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
+#endif
+	if (!has_selection)
 		return;
 
 	CDefaultFileExistsDlg dlg;
 	if (!dlg.Load(this, true))
 		return;
 
-	int downloadAction, uploadAction;
+	enum CFileExistsNotification::OverwriteAction downloadAction, uploadAction;
 	if (!dlg.Run(&downloadAction, &uploadAction))
 		return;
 
@@ -2527,6 +2581,8 @@ void CQueueView::OnTimer(wxTimerEvent& event)
 				pData->pEngine->Command(CDisconnectCommand());
 		}
 	}
+
+	event.Skip();
 }
 
 void CQueueView::DeleteEngines()
@@ -2561,6 +2617,12 @@ void CQueueView::WriteToFile(TiXmlElement* pElement) const
 
 void CQueueView::OnSetPriority(wxCommandEvent& event)
 {
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (!GetSelectedItemCount())
+		return;
+#endif
+
 	enum QueuePriority priority;
 
 	const int id = event.GetId();
