@@ -570,6 +570,10 @@ bool CRemoteListView::UpdateDirectoryListing(const CDirectoryListing *pDirectory
 			// sort order.
 			return false;
 		}
+
+		if (CFilterManager::HasActiveFilters())
+			return false;
+
 		wxASSERT(pDirectoryListing->GetCount() == m_pDirectoryListing->GetCount());
 		if (pDirectoryListing->GetCount() != m_pDirectoryListing->GetCount())
 			return false;
@@ -633,21 +637,7 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 		if (IsComparing())
 			ExitComparisonMode();
 
-		// Clear selection
-		int item = -1;
-		while (true)
-		{
-			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-			if (item == -1)
-				break;
-			SetSelection(item, false);
-		}
-		item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-		if (item != -1)
-		{
-			SetItemState(item, 0, wxLIST_STATE_FOCUSED);
-			wxASSERT(GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED) == -1);
-		}
+		ClearSelection();
 	}
 	else
 	{
@@ -676,6 +666,8 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 
 		m_indexMapping.push_back(m_pDirectoryListing->GetCount());
 
+		const wxString path = m_pDirectoryListing->path.GetPath();
+
 		CFilterManager filter;
 		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); i++)
 		{
@@ -685,7 +677,7 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 			data.icon = entry.dir ? m_dirIcon : -2;
 			m_fileData.push_back(data);
 
-			if (filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
+			if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
 			{
 				hidden++;
 				continue;
@@ -1021,6 +1013,28 @@ public:
 };
 typedef CReverseSort<CRemoteListViewSortPermissions> CRemoteListViewSortPermissions_Reverse;
 
+class CRemoteListViewSortOwnerGroup : public CRemoteListViewSort
+{
+public:
+	CRemoteListViewSortOwnerGroup(const CDirectoryListing* const pDirectoryListing, std::vector<CGenericFileData>& fileData, enum DirSortMode dirSortMode, CRemoteListView* const pRemoteListView)
+		: CRemoteListViewSort(pDirectoryListing, dirSortMode)
+	{
+	}
+
+	bool operator()(int a, int b) const
+	{
+		const CDirentry& data1 = (*m_pDirectoryListing)[a];
+		const CDirentry& data2 = (*m_pDirectoryListing)[b];
+
+		CMP(CmpDir, data1, data2);
+
+		CMP(CmpStringNoCase, data1.ownerGroup, data2.ownerGroup);
+
+		CMP_LESS(CmpName, data1, data2);
+	}
+};
+typedef CReverseSort<CRemoteListViewSortOwnerGroup> CRemoteListViewSortOwnerGroup_Reverse;
+
 void CRemoteListView::OnItemActivated(wxListEvent &event)
 {
 	if (!m_pState->IsRemoteIdle())
@@ -1096,6 +1110,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 			wxFileName fn = wxFileName(m_pState->GetLocalDir(), name);
 			m_pQueue->QueueFile(false, true, fn.GetFullPath(), name, m_pDirectoryListing->path, *pServer, entry.size);
+			m_pQueue->QueueFile_Finish(true);
 		}
 	}
 	else
@@ -1237,6 +1252,14 @@ void CRemoteListView::TransferSelectedFiles(const wxString& localDir, bool queue
 
 	wxASSERT(m_pState->LocalDirIsWriteable(localDir));
 
+	const CServer* pServer = m_pState->GetServer();
+	if (!pServer)
+	{
+		wxBell();
+		return;
+	}
+
+	bool added = false;
 	bool startRecursive = false;
 	long item = -1;
 	while (true)
@@ -1254,13 +1277,6 @@ void CRemoteListView::TransferSelectedFiles(const wxString& localDir, bool queue
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		const CServer* pServer = m_pState->GetServer();
-		if (!pServer)
-		{
-			wxBell();
-			return;
-		}
-
 		if (entry.dir)
 		{
 			if (!idle)
@@ -1279,8 +1295,11 @@ void CRemoteListView::TransferSelectedFiles(const wxString& localDir, bool queue
 		{
 			wxFileName fn = wxFileName(localDir, name);
 			m_pQueue->QueueFile(queueOnly, true, fn.GetFullPath(), name, m_pDirectoryListing->path, *pServer, entry.size);
+			added = true;
 		}
 	}
+	if (added)
+		m_pQueue->QueueFile_Finish(!queueOnly);
 
 	if (startRecursive)
 	{
@@ -1540,6 +1559,20 @@ void CRemoteListView::OnBeginLabelEdit(wxListEvent& event)
 
 void CRemoteListView::OnEndLabelEdit(wxListEvent& event)
 {
+	int item = event.GetIndex();
+#ifdef __WXMAC__
+	if (item != -1)
+	{
+		int from = item;
+		int to = item;
+		if (from)
+			from--;
+		if (to < GetItemCount() - 1)
+			to++;
+		RefreshItems(from, to);
+	}
+#endif
+
 	if (event.IsEditCancelled() || event.GetLabel() == _T(""))
 	{
 		event.Veto();
@@ -1560,7 +1593,6 @@ void CRemoteListView::OnEndLabelEdit(wxListEvent& event)
 		return;
 	}
 
-	int item = event.GetIndex();
 	if (!item)
 	{
 		event.Veto();
@@ -1790,13 +1822,15 @@ void CRemoteListView::ApplyCurrentFilter()
 	int totalDirCount = 0;
 	int hidden = 0;
 
+	const wxString path = m_pDirectoryListing->path.GetPath();
+
 	m_indexMapping.clear();
 	const unsigned int count = m_pDirectoryListing->GetCount();
 	m_indexMapping.push_back(count);
 	for (unsigned int i = 0; i < count; i++)
 	{
 		const CDirentry& entry = (*m_pDirectoryListing)[i];
-		if (filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
+		if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
 		{
 			hidden++;
 			continue;
@@ -1839,33 +1873,39 @@ std::list<wxString> CRemoteListView::RememberSelectedItems(wxString& focused)
 	wxASSERT(GetItemCount() == (int)m_indexMapping.size());
 	std::list<wxString> selectedNames;
 	// Remember which items were selected
-	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	while (item != -1)
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (GetSelectedItemCount())
+#endif
 	{
-		SetSelection(item, false);
-		if (!item)
+		int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+		while (item != -1)
 		{
-			selectedNames.push_back(_T(".."));
-			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-			continue;
-		}
-		int index = GetItemIndex(item);
-		if (index == -1 || m_fileData[index].flags == fill)
-		{
-			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-			continue;
-		}
-		const CDirentry& entry = (*m_pDirectoryListing)[index];
+			SetSelection(item, false);
+			if (!item)
+			{
+				selectedNames.push_back(_T(".."));
+				item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+				continue;
+			}
+			int index = GetItemIndex(item);
+			if (index == -1 || m_fileData[index].flags == fill)
+			{
+				item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+				continue;
+			}
+			const CDirentry& entry = (*m_pDirectoryListing)[index];
 
-		if (entry.dir)
-			selectedNames.push_back(_T("d") + entry.name);
-		else
-			selectedNames.push_back(_T("-") + entry.name);
+			if (entry.dir)
+				selectedNames.push_back(_T("d") + entry.name);
+			else
+				selectedNames.push_back(_T("-") + entry.name);
 
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+		}
 	}
 
-	item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
+	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 	if (item != -1)
 	{
 		int index = GetItemIndex(item);
@@ -2292,7 +2332,7 @@ void CRemoteListView::InitDateFormat()
 
 void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 {
-	if (!m_pState->IsRemoteConnected())
+	if (!m_pState->IsRemoteConnected() || !m_pDirectoryListing)
 	{
 		wxBell();
 		return;
@@ -2353,6 +2393,12 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 			wxBell();
 			return;
 		}
+
+		if (!m_pState->IsRemoteConnected() || !m_pDirectoryListing)
+		{
+			wxBell();
+			return;
+		}
 	}
 
 	CEditHandler::fileState state = pEditHandler->GetFileState(CEditHandler::remote, entry.name);
@@ -2373,15 +2419,36 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 		break;
 	case CEditHandler::edit:
 		{
-			int res = wxMessageBox(_("A file with that name is already being edited. Discard old file and download the new file?"), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
-			if (res != wxYES)
+			wxDialogEx dlg;
+			if (!dlg.Load(this, _T("ID_EDITEXISTING")))
 			{
 				wxBell();
 				return;
 			}
-			if (!pEditHandler->Remove(CEditHandler::remote, entry.name))
+			dlg.SetLabel(XRCID("ID_FILENAME"), entry.name);
+			if (dlg.ShowModal() != wxID_OK)
 			{
-				wxMessageBox(_("The selected file is still opened in some other program, please close it."), _("Selected file still being edited"), wxICON_EXCLAMATION);
+				wxBell();
+				return;
+			}
+
+			if (XRCCTRL(dlg, "ID_REOPEN", wxRadioButton)->GetValue())
+			{
+				pEditHandler->StartEditing(CEditHandler::remote, entry.name);
+				return;
+			}
+			else
+			{
+				if (!pEditHandler->Remove(CEditHandler::remote, entry.name))
+				{
+					wxMessageBox(_("The selected file is still opened in some other program, please close it."), _("Selected file still being edited"), wxICON_EXCLAMATION);
+					return;
+				}
+			}
+
+			if (!m_pState->IsRemoteConnected() || !m_pDirectoryListing)
+			{
+				wxBell();
 				return;
 			}
 		}
@@ -2399,6 +2466,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 
 	wxFileName fn = wxFileName(localDir, entry.name);
 	m_pQueue->QueueFile(false, true, fn.GetFullPath(), entry.name, m_pDirectoryListing->path, *m_pState->GetServer(), entry.size, CEditHandler::remote);
+	m_pQueue->QueueFile_Finish(true);
 }
 
 #ifdef __WXDEBUG__
@@ -2657,8 +2725,10 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortType(m_pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 3)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortTime(m_pDirectoryListing, m_fileData, dirSortMode, this));
-		else if (m_sortColumn == 3)
+		else if (m_sortColumn == 4)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortPermissions(m_pDirectoryListing, m_fileData, dirSortMode, this));
+		else if (m_sortColumn == 5)
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortOwnerGroup(m_pDirectoryListing, m_fileData, dirSortMode, this));
 		else
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortName(m_pDirectoryListing, m_fileData, dirSortMode, this));
 	}
@@ -2672,6 +2742,8 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortTime_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
 		else if (m_sortColumn == 4)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortPermissions_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
+		else if (m_sortColumn == 5)
+			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortOwnerGroup_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
 		else
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CRemoteListViewSortName_Reverse(m_pDirectoryListing, m_fileData, dirSortMode, this));
 	}
