@@ -573,10 +573,19 @@ checkmodifications_remote:
 		if (!mtime.IsValid())
 			continue;
 
-		if (iter->modificationTime.IsValid() & iter->modificationTime >= mtime)
+		if (iter->modificationTime.IsValid() && iter->modificationTime == mtime)
 			continue;
 
 		// File has changed, ask user what to do
+
+        wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
+		if (pTopWindow && pTopWindow->IsIconized())
+		{
+			pTopWindow->RequestUserAttention(wxUSER_ATTENTION_INFO);
+			insideCheckForModifications = false;
+			return;
+		}
+
 		CChangedFileDialog dlg;
 		if (!dlg.Load(wxTheApp->GetTopWindow(), _T("ID_CHANGEDFILE")))
 			continue;
@@ -586,13 +595,6 @@ checkmodifications_remote:
 
 		dlg.GetSizer()->Fit(&dlg);
 
-#ifdef __WXMSW__
-		// If mouse is captured, program
-		// could become unresponsive if showing
-		// dialog. Could happen during drag&drop.
-		ReleaseCapture();
-#endif
-
 		int res = dlg.ShowModal();
 
 		const bool remove = XRCCTRL(dlg, "ID_DELETE", wxCheckBox)->IsChecked();
@@ -640,12 +642,21 @@ checkmodifications_local:
 		if (!mtime.IsValid())
 			continue;
 
-		if (iter->modificationTime.IsValid() & iter->modificationTime >= mtime)
+		if (iter->modificationTime.IsValid() && iter->modificationTime == mtime)
 			continue;
 
 		// File has changed, ask user what to do
+
+		wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
+		if (pTopWindow && pTopWindow->IsIconized())
+		{
+			pTopWindow->RequestUserAttention(wxUSER_ATTENTION_INFO);
+			insideCheckForModifications = false;
+			return;
+		}
+
 		CChangedFileDialog dlg;
-		if (!dlg.Load(wxTheApp->GetTopWindow(), _T("ID_CHANGEDFILE")))
+		if (!dlg.Load(pTopWindow, _T("ID_CHANGEDFILE")))
 			continue;
 		XRCCTRL(dlg, "ID_DESC_UPLOAD_REMOTE", wxStaticText)->Hide();
 		XRCCTRL(dlg, "ID_DELETE", wxCheckBox)->SetLabel(_("&Finish editing"));
@@ -659,8 +670,15 @@ checkmodifications_local:
 
 		if (res == wxID_YES)
 			UploadFile(local, iter->name, remove);
-		else
+		else if (remove)
 			m_fileDataList[local].erase(iter);
+		else if (!fn.FileExists())
+		{
+			m_fileDataList[remote].erase(iter);
+			goto checkmodifications_remote;
+		}
+		else
+			iter->modificationTime = mtime;
 		goto checkmodifications_local;
 	}
 
@@ -705,6 +723,7 @@ bool CEditHandler::UploadFile(enum CEditHandler::fileType type, const wxString&
 	wxULongLong size = fn.GetSize();
 	
 	m_pQueue->QueueFile(false, false, fn.GetFullPath(), fn.GetFullName(), iter->remotePath, iter->server, wxLongLong(size.GetHi(), size.GetLo()), type);
+	m_pQueue->QueueFile_Finish(true);
 
 	return true;
 }
