@@ -47,6 +47,11 @@ typedef struct fz_tagNMLVODSTATECHANGE
 } fzNMLVODSTATECHANGE;
 #pragma pack(pop)
 
+#ifndef LVN_MARQUEEBEGIN
+// MinGW lacks this constant
+#define LVN_MARQUEEBEGIN        (LVN_FIRST-56)
+#endif
+
 template<class CFileData> LRESULT CALLBACK CFileListCtrl<CFileData>::WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	std::map<HWND, char*>::iterator iter = m_hwnd_map.find(hWnd);
@@ -106,6 +111,10 @@ template<class CFileData> LRESULT CALLBACK CFileListCtrl<CFileData>::WindowProc(
 			pFileListCtrl->m_pFilelistStatusBar->UnselectAll();
 		}
 	}
+	else if (pNmhdr->code == LVN_MARQUEEBEGIN)
+	{
+		pFileListCtrl->SetFocus();
+	}
 
 	return CallWindowProc(pFileListCtrl->m_prevWndproc, hWnd, uMsg, wParam, lParam);
 }
@@ -239,9 +248,15 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		selected = new bool[m_fileData.size()];
 		memset(selected, 0, sizeof(bool) * m_fileData.size());
 
-		int item = -1;
-		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-			selected[m_indexMapping[item]] = 1;
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+		if (GetSelectedItemCount())		
+#endif
+		{
+			int item = -1;
+			while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+				selected[m_indexMapping[item]] = 1;
+		}
 		focused = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 		if (focused != -1)
 			focused = m_indexMapping[focused];
@@ -540,13 +555,19 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 	}
 	m_comparisonSelections.push_back(focus);
 
-	int item = -1;
-	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (GetSelectedItemCount())
+#endif
 	{
-		int index = m_indexMapping[item];
-		if (m_fileData[index].flags == fill)
-			continue;
-		m_comparisonSelections.push_back(index);
+		int item = -1;
+		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+		{
+			int index = m_indexMapping[item];
+			if (m_fileData[index].flags == fill)
+				continue;
+			m_comparisonSelections.push_back(index);
+		}
 	}
 }
 
@@ -833,3 +854,23 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxC
 	}
 }
 #endif
+
+template<class CFileData> void CFileListCtrl<CFileData>::ClearSelection()
+{
+	// Clear selection
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	if (GetSelectedItemCount())
+#endif
+	{
+		int item = -1;
+		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+		{
+			SetSelection(item, false);
+		}
+	}
+
+	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
+	if (item != -1)
+		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
+}
