@@ -3,7 +3,6 @@
 #include "filteredit.h"
 #include "ipcmutex.h"
 #include "filezillaapp.h"
-#include "../tinyxml/tinyxml.h"
 #include "xmlfunctions.h"
 #include <wx/regex.h>
 #include "Mainfrm.h"
@@ -14,6 +13,7 @@ bool CFilterManager::m_loaded = false;
 std::vector<CFilter> CFilterManager::m_globalFilters;
 std::vector<CFilterSet> CFilterManager::m_globalFilterSets;
 unsigned int CFilterManager::m_globalCurrentFilterSet = 0;
+bool CFilterManager::m_filters_disabled = false;
 
 BEGIN_EVENT_TABLE(CFilterDialog, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CFilterDialog::OnOK)
@@ -227,6 +227,8 @@ void CFilterDialog::SaveFilters()
 
 	SaveXmlFile(file, pDocument);
 	delete pDocument->GetDocument();
+
+	m_filters_disabled = false;
 }
 
 void CFilterDialog::DisplayFilters()
@@ -430,6 +432,10 @@ CFilterManager::CFilterManager()
 {
 	m_currentFilterSet = 0;
 	LoadFilters();
+	m_filters = m_globalFilters;
+	m_filterSets = m_globalFilterSets;
+	m_currentFilterSet = m_globalCurrentFilterSet;
+
 	CompileRegexes();
 
 	if (m_globalFilterSets.empty())
@@ -443,10 +449,19 @@ CFilterManager::CFilterManager()
 	}
 }
 
-bool CFilterManager::HasActiveFilters() const
+bool CFilterManager::HasActiveFilters(bool ignore_disabled /*=false*/)
 {
+	if (!m_loaded)
+		LoadFilters();
+
+	if (m_globalFilterSets.empty())
+		return false;
+
 	wxASSERT(m_globalCurrentFilterSet < m_globalFilterSets.size());
 
+	if (m_filters_disabled && !ignore_disabled)
+		return false;
+
 	const CFilterSet& set = m_globalFilterSets[m_globalCurrentFilterSet];
 	for (unsigned int i = 0; i < m_globalFilters.size(); i++)
 	{
@@ -477,8 +492,11 @@ bool CFilterManager::HasSameLocalAndRemoteFilters() const
 	return true;
 }
 
-bool CFilterManager::FilenameFiltered(const wxString& name, bool dir, wxLongLong size, bool local, int attributes) const
+bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes) const
 {
+	if (m_filters_disabled)
+		return false;
+
 	wxASSERT(m_currentFilterSet < m_filterSets.size());
 
 	const CFilterSet& set = m_filterSets[m_currentFilterSet];
@@ -489,13 +507,13 @@ bool CFilterManager::FilenameFiltered(const wxString& name, bool dir, wxLongLong
 		if (local)
 		{
 			if (set.local[i])
-				if (FilenameFilteredByFilter(name, dir, size, i, attributes))
+				if (FilenameFilteredByFilter(name, path, dir, size, i, attributes))
 					return true;
 		}
 		else
 		{
 			if (set.remote[i])
-				if (FilenameFilteredByFilter(name, dir, size, i, attributes))
+				if (FilenameFilteredByFilter(name, path, dir, size, i, attributes))
 					return true;
 		}
 	}
@@ -503,7 +521,7 @@ bool CFilterManager::FilenameFiltered(const wxString& name, bool dir, wxLongLong
 	return false;
 }
 
-bool CFilterManager::FilenameFilteredByFilter(const wxString& name, bool dir, wxLongLong size, unsigned int filterIndex, int attributes) const
+bool CFilterManager::FilenameFilteredByFilter(const wxString& name, const wxString& path, bool dir, wxLongLong size, unsigned int filterIndex, int attributes) const
 {
 	wxRegEx regex;
 	const CFilter& filter = m_filters[filterIndex];
@@ -583,6 +601,69 @@ bool CFilterManager::FilenameFilteredByFilter(const wxString& name, bool dir, wx
 					match = true;
 			}
 			break;
+		case (enum t_filterType)::path:
+			switch (condition.condition)
+			{
+			case 0:
+				if (filter.matchCase)
+				{
+					if (path.Contains(condition.strValue))
+						match = true;
+				}
+				else
+				{
+					if (path.Lower().Contains(condition.strValue.Lower()))
+						match = true;
+				}
+				break;
+			case 1:
+				if (filter.matchCase)
+				{
+					if (path == condition.strValue)
+						match = true;
+				}
+				else
+				{
+					if (!path.CmpNoCase(condition.strValue))
+						match = true;
+				}
+				break;
+			case 2:
+				{
+					const wxString& left = path.Left(condition.strValue.Len());
+					if (filter.matchCase)
+					{
+						if (left == condition.strValue)
+							match = true;
+					}
+					else
+					{
+						if (!left.CmpNoCase(condition.strValue))
+							match = true;
+					}
+				}
+				break;
+			case 3:
+				{
+					const wxString& right = path.Right(condition.strValue.Len());
+					if (filter.matchCase)
+					{
+						if (right == condition.strValue)
+							match = true;
+					}
+					else
+					{
+						if (!right.CmpNoCase(condition.strValue))
+							match = true;
+					}
+				}
+				break;
+			case 4:
+				wxASSERT(condition.pRegEx);
+				if (condition.pRegEx && condition.pRegEx->Matches(path))
+					match = true;
+			}
+			break;
 		case (enum t_filterType)::size:
 			if (size == -1)
 				continue;
@@ -718,7 +799,7 @@ bool CFilterManager::CompileRegexes()
 		{
 			CFilterCondition& condition = *iter;
 			delete condition.pRegEx;
-			if (!condition.type && condition.condition == 4)
+			if ((condition.type == name || condition.type == path) && condition.condition == 4)
 				condition.pRegEx = new wxRegEx(condition.strValue);
 			else
 				condition.pRegEx = 0;
@@ -730,12 +811,8 @@ bool CFilterManager::CompileRegexes()
 void CFilterManager::LoadFilters()
 {
 	if (m_loaded)
-	{
-		m_filters = m_globalFilters;
-		m_filterSets = m_globalFilterSets;
-		m_currentFilterSet = m_globalCurrentFilterSet;
 		return;
-	}
+
 	m_loaded = true;
 
 	CInterProcessMutex mutex(MUTEX_FILTERS);
@@ -844,7 +921,6 @@ void CFilterManager::LoadFilters()
 
 		pFilter = pFilter->NextSiblingElement("Filter");
 	}
-	m_filters = m_globalFilters;
 
 	TiXmlElement* pSets = pDocument->FirstChildElement("Sets");
 	if (!pSets)
@@ -874,10 +950,9 @@ void CFilterManager::LoadFilters()
 				continue;
 		}
 
-		if (set.local.size() == m_filters.size())
+		if (set.local.size() == m_globalFilters.size())
 			m_globalFilterSets.push_back(set);
 	}
-	m_filterSets = m_globalFilterSets;
 
 	wxString attribute = GetTextAttribute(pSets, "Current");
 	unsigned long value;
@@ -887,7 +962,17 @@ void CFilterManager::LoadFilters()
 			m_globalCurrentFilterSet = value;
 	}
 
-	m_currentFilterSet = m_globalCurrentFilterSet;
-
 	delete pDocument->GetDocument();
 }
+
+void CFilterManager::ToggleFilters()
+{
+	if (m_filters_disabled)
+	{
+		m_filters_disabled = false;
+		return;
+	}
+
+	if (HasActiveFilters(true))
+		m_filters_disabled = true;
+}
