@@ -10,6 +10,7 @@ enum t_filterType
 	size,
 	attributes,
 	permissions,
+	path,
 
 	filterType_size
 };
@@ -69,16 +70,18 @@ public:
 	CFilterManager();
 
 	// Note: Under non-windows, attributes are permissions
-	bool FilenameFiltered(const wxString& name, bool dir, wxLongLong size, bool local, int attributes) const;
-	bool HasActiveFilters() const;
+	bool FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes) const;
+	static bool HasActiveFilters(bool ignore_disabled = false);
 
 	bool HasSameLocalAndRemoteFilters() const;
 
+	static void ToggleFilters();
+
 protected:
 	bool CompileRegexes();
-	bool FilenameFilteredByFilter(const wxString& name, bool dir, wxLongLong size, unsigned int filterIndex, int attributes) const;
+	bool FilenameFilteredByFilter(const wxString& name, const wxString& path, bool dir, wxLongLong size, unsigned int filterIndex, int attributes) const;
 
-	void LoadFilters();
+	static void LoadFilters();
 	static bool m_loaded;
 
 	static std::vector<CFilter> m_globalFilters;
@@ -88,6 +91,8 @@ protected:
 	std::vector<CFilterSet> m_filterSets;
 	unsigned int m_currentFilterSet;
 	static unsigned int m_globalCurrentFilterSet;
+
+	static bool m_filters_disabled;
 };
 
 class CMainFrame;
