@@ -265,7 +265,7 @@ CFileItem::CFileItem(CServerItem* parent, bool queued, bool download, const wxSt
 	m_errorCount = 0;
 	m_remove = false;
 	m_pEngineData = 0;
-	m_defaultFileExistsAction = -1;
+	m_defaultFileExistsAction = CFileExistsNotification::unknown;
 	m_edit = CEditHandler::none;
 	m_madeProgress = false;
 	m_autoResume = false;
@@ -420,7 +420,7 @@ void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
 	wxFAIL_MSG(_T("File item not deleted from m_fileList"));
 }
 
-void CServerItem::SetDefaultFileExistsAction(int action, const enum TransferDirection direction)
+void CServerItem::SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction)
 {
 	for (std::vector<CQueueItem*>::iterator iter = m_children.begin(); iter != m_children.end(); iter++)
 	{
@@ -693,7 +693,7 @@ CFolderScanItem::CFolderScanItem(CServerItem* parent, bool queued, bool download
 	pair.remotePath.SetSafePath(remotePath.GetSafePath().c_str());
 	m_dirsToCheck.push_back(pair);
 
-	m_defaultFileExistsAction = -1;
+	m_defaultFileExistsAction = CFileExistsNotification::unknown;
 }
 
 bool CFolderScanItem::TryRemoveAll()
@@ -714,6 +714,7 @@ EVT_ERASE_BACKGROUND(CQueueViewBase::OnEraseBackground)
 EVT_NAVIGATION_KEY(CQueueViewBase::OnNavigationKey)
 EVT_CHAR(CQueueViewBase::OnChar)
 EVT_LIST_COL_END_DRAG(wxID_ANY, CQueueViewBase::OnEndColumnDrag)
+EVT_TIMER(wxID_ANY, CQueueViewBase::OnTimer)
 END_EVENT_TABLE()
 
 CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
@@ -740,6 +741,8 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"),  wxART_OTHER, wxSize(16, 16)));
 
 	AssignImageList(pImageList, wxIMAGE_LIST_SMALL);
+
+	m_filecount_delay_timer.SetOwner(this);
 }
 
 CQueueViewBase::~CQueueViewBase()
@@ -980,10 +983,15 @@ void CQueueViewBase::UpdateSelections_ItemAdded(int added)
 	// selections. Though worst case is still O(n), as with every algorithm to
 	// move selections.
 
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	const int selection_count = GetSelectedItemCount();
+	if (!selection_count)
+		return;
+#endif
+
 	// Go through all items, keep record of the previous selected item
-	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	while (item != -1 && item < added)
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	int item = GetNextItem(added - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
 	int prevItem = -1;
 	while (item != -1)
@@ -1019,12 +1027,17 @@ void CQueueViewBase::UpdateSelections_ItemAdded(int added)
 
 void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 {
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	const int selection_count = GetSelectedItemCount();
+	if (!selection_count)
+		return;
+#endif
+
 	std::list<int> itemsToSelect;
 
 	// Go through all selected items
-	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	while (item != -1 && item < added)
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	int item = GetNextItem(added - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
 	while (item != -1)
 	{
@@ -1049,13 +1062,18 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 
 void CQueueViewBase::UpdateSelections_ItemRemoved(int removed)
 {
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	const int selection_count = GetSelectedItemCount();
+	if (!selection_count)
+		return;
+#endif
+
 	SetItemState(removed, 0, wxLIST_STATE_SELECTED);
 
-	int prevItem = -1;
-	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	while (item != -1 && item < removed)
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	int item = GetNextItem(removed - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
+	int prevItem = -1;
 	while (item != -1)
 	{
 		if (prevItem != -1)
@@ -1086,13 +1104,18 @@ void CQueueViewBase::UpdateSelections_ItemRemoved(int removed)
 
 void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 {
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	const int selection_count = GetSelectedItemCount();
+	if (!selection_count)
+		return;
+#endif
+
 	SetItemState(removed, 0, wxLIST_STATE_SELECTED);
 
 	std::list<int> itemsToUnselect;
 
-	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	while (item != -1 && item < removed)
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	int item = GetNextItem(removed - 1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 
 	while (item != -1)
 	{
@@ -1185,6 +1208,12 @@ void CQueueViewBase::CommitChanges()
 
 void CQueueViewBase::DisplayNumberQueuedFiles()
 {
+	if (m_filecount_delay_timer.IsRunning())
+	{
+		m_fileCountChanged = true;
+		return;
+	}
+
 	wxString str;
 	if (m_fileCount > 0)
 	{
@@ -1204,15 +1233,19 @@ void CQueueViewBase::DisplayNumberQueuedFiles()
 
 	m_fileCountChanged = false;
 	m_folderScanCountChanged = false;
+
+	m_filecount_delay_timer.Start(200, true);
 }
 
 void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 {
+	const int newIndex = GetItemIndex(pServerItem) + pServerItem->GetChildrenCount(true) + 1;
+
 	pServerItem->AddChild(pItem);
 	m_itemCount++;
 
 	if (m_insertionStart == -1)
-		m_insertionStart = GetItemIndex(pItem);
+		m_insertionStart = newIndex;
 	m_insertionCount++;
 
 	if (pItem->GetType() == QueueItemType_File || pItem->GetType() == QueueItemType_Folder)
@@ -1359,6 +1392,18 @@ void CQueueViewBase::OnEndColumnDrag(wxListEvent& event)
 	}
 }
 
+void CQueueViewBase::OnTimer(wxTimerEvent& event)
+{
+	if (event.GetId() != m_filecount_delay_timer.GetId())
+	{
+		event.Skip();
+		return;
+	}
+
+	if (m_fileCountChanged || m_folderScanCountChanged)
+		DisplayNumberQueuedFiles();
+}
+
 // ------
 // CQueue
 // ------
