@@ -20,6 +20,7 @@
 #include "optionspage_dateformatting.h"
 #include "optionspage_sizeformatting.h"
 #include "optionspage_edit.h"
+#include "optionspage_proxy.h"
 #include "filezillaapp.h"
 #include "Mainfrm.h"
 
@@ -32,6 +33,7 @@ enum pagenames
 	page_connection_passive,
 	page_connection_ftp_proxy,
 	page_connection_sftp,
+	page_connection_proxy,
 	page_transfer,
 	page_filetype,
 	page_fileexists,
@@ -113,6 +115,7 @@ bool CSettingsDialog::LoadPages()
 	ADD_PAGE(_("Passive mode"), COptionsPageConnectionPassive, page_connection_ftp);
 	ADD_PAGE(_("FTP Proxy"), COptionsPageFtpProxy, page_connection_ftp);
 	ADD_PAGE(_("SFTP"), COptionsPageConnectionSFTP, page_connection);
+	ADD_PAGE(_("Generic proxy"), COptionsPageProxy, page_connection);
 	ADD_PAGE(_("Transfers"), COptionsPageTransfer, page_none);
 	ADD_PAGE(_("File Types"), COptionsPageFiletype, page_transfer);
 	ADD_PAGE(_("File exists action"), COptionsPageFileExists, page_transfer);
@@ -123,7 +126,10 @@ bool CSettingsDialog::LoadPages()
 	ADD_PAGE(_("Language"), COptionsPageLanguage, page_none);
 	ADD_PAGE(_("File editing"), COptionsPageEdit, page_none);
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	ADD_PAGE(_("Update Check"), COptionsPageUpdateCheck, page_none);
+	if (!COptions::Get()->GetDefaultVal(DEFAULT_DISABLEUPDATECHECK))
+	{
+		ADD_PAGE(_("Update Check"), COptionsPageUpdateCheck, page_none);
+	}
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
 	ADD_PAGE(_("Logging"), COptionsPageLogging, page_none);
 	ADD_PAGE(_("Debug"), COptionsPageDebug, page_none);
@@ -207,7 +213,7 @@ bool CSettingsDialog::LoadPages()
 	if (!m_activePanel)
 	{
 		m_activePanel = m_pages[0].page;
-		m_activePanel->Show();
+		m_activePanel->Display();
 	}
 
 	return true;
@@ -237,7 +243,7 @@ void CSettingsDialog::OnPageChanged(wxTreeEvent& event)
 		if (m_pages[i].id == item)
 		{
 			m_activePanel = m_pages[i].page;
-			m_activePanel->Show();
+			m_activePanel->Display();
 			break;
 		}
 	}
