@@ -1,7 +1,6 @@
 #include "FileZilla.h"
 #include "sitemanager.h"
 #include "Options.h"
-#include "../tinyxml/tinyxml.h"
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
 #include "ipcmutex.h"
@@ -9,6 +8,9 @@
 #include "conditionaldialog.h"
 #include "window_state_manager.h"
 #include <wx/dnd.h>
+#ifdef __WXMSW__
+#include "commctrl.h"
+#endif
 
 std::map<int, CSiteManagerItemData*> CSiteManager::m_idMap;
 
@@ -288,6 +290,23 @@ bool CSiteManager::Create(wxWindow* parent, const CServer* pServer /*=0*/)
 
 	pTree->AssignImageList(pImageList);
 
+#ifdef __WXMSW__
+	// Make pages at least wide enough to fit all tabs
+	wxNotebook *pBook = XRCCTRL(*this, "ID_NOTEBOOK", wxNotebook);
+	
+	HWND hWnd = (HWND)pBook->GetHandle();
+
+	int width = 4;
+	for (unsigned int i = 0; i < pBook->GetPageCount(); i++)
+	{
+		RECT tab_rect;
+		TabCtrl_GetItemRect(hWnd, i, &tab_rect);
+		width += tab_rect.right - tab_rect.left;
+	}
+	int margin = pBook->GetSize().x - pBook->GetPage(0)->GetSize().x;
+	pBook->GetPage(0)->GetSizer()->SetMinSize(wxSize(width - margin, 0));
+#endif
+
 	Layout();
 	wxGetApp().GetWrapEngine()->WrapRecursive(this, 1.33, "Site Manager");
 
@@ -786,7 +805,7 @@ void CSiteManager::OnNewFolder(wxCommandEvent&event)
 	pTree->EditLabel(newItem);
 }
 
-static enum ServerType GetTypeFromName(const wxString& name)
+static enum ServerType GetServerTypeFromName(const wxString& name)
 {
 	for (int i = 0; i < SERVERTYPE_MAX; i++)
 	{
@@ -798,6 +817,20 @@ static enum ServerType GetTypeFromName(const wxString& name)
 	return DEFAULT;
 }
 
+static enum LogonType GetLogonTypeFromName(const wxString& name)
+{
+	if (name == _("Normal"))
+		return NORMAL;
+	else if (name == _("Ask for password"))
+		return ASK;
+	else if (name == _("Interactive"))
+		return INTERACTIVE;
+	else if (name == _("Account"))
+		return ACCOUNT;
+	else
+		return ANONYMOUS;
+}
+
 bool CSiteManager::Verify()
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
@@ -820,17 +853,31 @@ bool CSiteManager::Verify()
 		return false;
 	}
 
+	enum LogonType logon_type = GetLogonTypeFromName(XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection());
+
 	wxString protocolName = XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->GetStringSelection();
 	enum ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
 	if (protocol == SFTP &&
-		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection() == _("Account"))
+		logon_type == ACCOUNT)
 	{
 		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
 		wxMessageBox(_("'Account' logontype not supported by selected protocol"));
 		return false;
 	}
 
+	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) != 0 &&
+		(logon_type == ACCOUNT || logon_type == NORMAL))
+	{
+		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
+		wxMessageBox(_("FileZilla is running in kiosk mode.\n'Normal' and 'Account' logontypes are not available in this mode."));
+		return false;
+	}
+
 	CServer server;
+
+	// Set selected type
+	server.SetLogonType(logon_type);
+
 	if (protocol != UNKNOWN)
 		server.SetProtocol(protocol);
 
@@ -845,7 +892,7 @@ bool CSiteManager::Verify()
 		return false;
 	}
 
-	XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetValue(server.GetHost());
+	XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetValue(server.FormatHost(true));
 	XRCCTRL(*this, "ID_PORT", wxTextCtrl)->SetValue(wxString::Format(_T("%d"), server.GetPort()));
 
 	protocolName = CServer::GetProtocolName(server.GetProtocol());
@@ -863,17 +910,41 @@ bool CSiteManager::Verify()
 		}
 	}
 
-	// Require username for non-anonymous logon type
-	if (XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection() != _("Anonymous") &&
-		XRCCTRL(*this, "ID_USER", wxTextCtrl)->GetValue() == _T(""))
+	// Require username for non-anonymous, non-ask logon type
+	const wxString user = XRCCTRL(*this, "ID_USER", wxTextCtrl)->GetValue();
+	if (logon_type != ANONYMOUS &&
+		logon_type != ASK &&
+		logon_type != INTERACTIVE &&
+		user == _T(""))
 	{
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetFocus();
 		wxMessageBox(_("You have to specify a user name"));
 		return false;
 	}
 
+	// The way TinyXML handles blanks, we can't use username of only spaces
+	if (user != _T(""))
+	{
+		bool space_only = true;
+		for (unsigned int i = 0; i < user.Len(); i++)
+		{
+			if (user[i] != ' ')
+			{
+				space_only = false;
+				break;
+			}
+		}
+		if (space_only)
+		{
+			XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetFocus();
+			wxMessageBox(_("Username cannot be a series of spaces"));
+			return false;
+		}
+
+	}
+
 	// Require account for account logon type
-	if (XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection() == _("Account") &&
+	if (logon_type == ACCOUNT &&
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->GetValue() == _T(""))
 	{
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->SetFocus();
@@ -886,7 +957,7 @@ bool CSiteManager::Verify()
 	{
 		CServerPath remotePath;
 		const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
-		remotePath.SetType(GetTypeFromName(serverType));
+		remotePath.SetType(GetServerTypeFromName(serverType));
 		if (!remotePath.SetPath(remotePathRaw))
 		{
 			XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetFocus();
@@ -1090,7 +1161,14 @@ bool CSiteManager::UpdateServer()
 
 	unsigned long port;
 	XRCCTRL(*this, "ID_PORT", wxTextCtrl)->GetValue().ToULong(&port);
-	data->m_server.SetHost(XRCCTRL(*this, "ID_HOST", wxTextCtrl)->GetValue(), port);
+	wxString host = XRCCTRL(*this, "ID_HOST", wxTextCtrl)->GetValue();
+	// SetHost does not accept URL syntax
+	if (host[0] == '[')
+	{
+		host.RemoveLast();
+		host = host.Mid(1);
+	}
+	data->m_server.SetHost(host, port);
 
 	const wxString& protocolName = XRCCTRL(*this, "ID_PROTOCOL", wxChoice)->GetStringSelection();
 	const enum ServerProtocol protocol = CServer::GetProtocolFromName(protocolName);
@@ -1099,17 +1177,8 @@ bool CSiteManager::UpdateServer()
 	else
 		data->m_server.SetProtocol(FTP);
 
-	const wxString logonType = XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection();
-	if (logonType == _("Normal"))
-		data->m_server.SetLogonType(NORMAL);
-	else if (logonType == _("Ask for password"))
-		data->m_server.SetLogonType(ASK);
-	else if (logonType == _("Interactive"))
-		data->m_server.SetLogonType(INTERACTIVE);
-	else if (logonType == _("Account"))
-		data->m_server.SetLogonType(ACCOUNT);
-	else
-		data->m_server.SetLogonType(ANONYMOUS);
+	enum LogonType logon_type = GetLogonTypeFromName(XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->GetStringSelection());
+	data->m_server.SetLogonType(logon_type);
 
 	data->m_server.SetUser(XRCCTRL(*this, "ID_USER", wxTextCtrl)->GetValue(),
 						   XRCCTRL(*this, "ID_PASS", wxTextCtrl)->GetValue());
@@ -1118,7 +1187,7 @@ bool CSiteManager::UpdateServer()
 	data->m_comments = XRCCTRL(*this, "ID_COMMENTS", wxTextCtrl)->GetValue();
 
 	const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
-	data->m_server.SetType(GetTypeFromName(serverType));
+	data->m_server.SetType(GetServerTypeFromName(serverType));
 
 	data->m_localDir = XRCCTRL(*this, "ID_LOCALDIR", wxTextCtrl)->GetValue();
 	data->m_remoteDir = CServerPath();
@@ -1296,7 +1365,7 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_CONNECT", wxWindow)->Enable(true);
 
 		XRCCTRL(*this, "ID_HOST", wxWindow)->Enable(!predefined);
-		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetValue(data->m_server.GetHost());
+		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetValue(data->m_server.FormatHost(true));
 		unsigned int port = data->m_server.GetPort();
 
 		if (port != CServer::GetDefaultPort(data->m_server.GetProtocol()))
@@ -1314,7 +1383,7 @@ void CSiteManager::SetCtrlState()
 		XRCCTRL(*this, "ID_BYPASSPROXY", wxCheckBox)->SetValue(data->m_server.GetBypassProxy());
 
 		XRCCTRL(*this, "ID_USER", wxTextCtrl)->Enable(!predefined && data->m_server.GetLogonType() != ANONYMOUS);
-		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->Enable(!predefined && data->m_server.GetLogonType() == NORMAL || data->m_server.GetLogonType() == ACCOUNT);
+		XRCCTRL(*this, "ID_PASS", wxTextCtrl)->Enable(!predefined && (data->m_server.GetLogonType() == NORMAL || data->m_server.GetLogonType() == ACCOUNT));
 		XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->Enable(!predefined && data->m_server.GetLogonType() == ACCOUNT);
 
 		switch (data->m_server.GetLogonType())
@@ -1567,8 +1636,15 @@ public:
 			const wxMenuItem* const pItem = pMenu->FindItemByPosition(i);
 			if (!pItem)
 				continue;
+
+			// Use same sorting as site tree in site manager
+#ifdef __WXMSW__
+			if (pItem->GetLabel().CmpNoCase(name) > 0)
+				break;
+#else
 			if (pItem->GetLabel() > name)
 				break;
+#endif
 		}
 
 		return i;
@@ -1782,7 +1858,12 @@ void CSiteManager::OnBeginDrag(wxTreeEvent& event)
 	}
 
 	const bool predefined = IsPredefinedItem(item);
-	const bool root_or_predefined = (item == pTree->GetRootItem() || item == m_ownSites || predefined);
+	const bool root = item == pTree->GetRootItem() || item == m_ownSites;
+	if (root)
+	{
+		event.Veto();
+		return;
+	}
 
 	CSiteManagerDataObject obj;
 
@@ -1791,7 +1872,7 @@ void CSiteManager::OnBeginDrag(wxTreeEvent& event)
 
 	m_dropSource = item;
 
-	source.DoDragDrop(root_or_predefined ? wxDrag_CopyOnly : wxDrag_DefaultMove);
+	source.DoDragDrop(predefined ? wxDrag_CopyOnly : wxDrag_DefaultMove);
 
 	m_dropSource = wxTreeItemId();
 }
