@@ -415,14 +415,15 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
     if (restart) {
 	char decbuf[30];
 	if (seek_file(file, uint64_make(0,0) , FROM_END) == -1) {
+	    close_wfile(file);
 	    fzprintf(sftpError, "reget: cannot restart %s - file too large",
 		   outfname);
-	    	sftp_register(req = fxp_close_send(fh));
-		rreq = sftp_find_request(pktin = sftp_recv());
-		assert(rreq == req);
-		fxp_close_recv(pktin, rreq);
-		
-		return 0;
+	    sftp_register(req = fxp_close_send(fh));
+	    rreq = sftp_find_request(pktin = sftp_recv());
+	    assert(rreq == req);
+	    fxp_close_recv(pktin, rreq);
+
+	    return 0;
 	}
 	    
 	offset = get_file_posn(file);
@@ -649,6 +650,7 @@ int sftp_put_file(char *fname, char *outfname, int recurse, int restart)
     fh = fxp_open_recv(pktin, rreq);
 
     if (!fh) {
+	close_rfile(file);
 	fzprintf(sftpError, "%s: open for write: %s", outfname, fxp_error());
 	return 0;
     }
@@ -664,10 +666,12 @@ int sftp_put_file(char *fname, char *outfname, int recurse, int restart)
 	ret = fxp_fstat_recv(pktin, rreq, &attrs);
 
 	if (!ret) {
+	    close_rfile(file);
 	    fzprintf(sftpError, "read size of %s: %s", outfname, fxp_error());
 	    return 0;
 	}
 	if (!(attrs.flags & SSH_FILEXFER_ATTR_SIZE)) {
+	    close_rfile(file);
 	    fzprintf(sftpError, "read size of %s: size was not given", outfname);
 	    return 0;
 	}
@@ -708,7 +712,7 @@ int sftp_put_file(char *fname, char *outfname, int recurse, int restart)
 	if (!xfer_done(xfer)) {
 	    pktin = sftp_recv();
 	    ret = xfer_upload_gotpkt(xfer, pktin);
-	    if (!ret) {
+	    if (ret <= 0 && !err) {
 		fzprintf(sftpError, "error while writing: %s\n", fxp_error());
 		err = 1;
 	    }
@@ -975,6 +979,77 @@ int sftp_cmd_keyfile(struct sftp_command *cmd)
     return 1;
 }
 
+int sftp_cmd_proxy(struct sftp_command *cmd)
+{
+    int proxy_type;
+    int portnumber;
+
+    if (cmd->nwords < 2) {
+	fzprintf(sftpError, "Not enough arguments to proxy command");
+	return 0;
+    }
+
+    if (!strcmp(cmd->words[1], "0")) {
+	cfg.proxy_type = PROXY_NONE;
+        fznotify1(sftpDone, 1);
+	return 1;
+    }
+    if (!strcmp(cmd->words[1], "1")) {
+	proxy_type = PROXY_HTTP;
+    }
+    else if (!strcmp(cmd->words[1], "2")) {
+	proxy_type = PROXY_SOCKS5;
+    }
+    else {
+	fzprintf(sftpError, "Unknown proxy type");
+	return 0;
+    }
+
+    if (cmd->nwords < 4) {
+	fzprintf(sftpError, "Not enough arguments to proxy command");
+	return 0;
+    }
+
+    portnumber = atoi(cmd->words[3]);
+    if (portnumber < 0 || portnumber > 65535) {
+	fzprintf(sftpError, "Invalid port");
+	return 0;
+    }
+
+    if (strlen(cmd->words[2]) >= sizeof(cfg.proxy_host)) {
+	fzprintf(sftpError,  "Host too long");
+	return 0;
+    }
+    if (cmd->nwords > 4 && strlen(cmd->words[4]) >= sizeof(cfg.proxy_username)) {
+	fzprintf(sftpError,  "User too long");
+	return 0;
+    }
+    if (cmd->nwords > 5 && strlen(cmd->words[5]) >= sizeof(cfg.proxy_password)) {
+	fzprintf(sftpError,  "Password too long");
+	return 0;
+    }
+
+    if (cmd->nwords > 5) {
+	strcpy(cfg.proxy_username, cmd->words[4]);
+	strcpy(cfg.proxy_password, cmd->words[5]);
+    }
+    else if (cmd->nwords > 4) {
+	strcpy(cfg.proxy_username, cmd->words[4]);
+	cfg.proxy_password[0] = 0;
+    }
+    else {
+	cfg.proxy_username[0] = 0;
+	cfg.proxy_password[0] = 0;
+    }
+
+    cfg.proxy_type = proxy_type;
+    strcpy(cfg.proxy_host, cmd->words[2]);
+    cfg.proxy_port = portnumber;
+
+    fznotify1(sftpDone, 1);
+    return 1;
+}
+
 int sftp_cmd_close(struct sftp_command *cmd)
 {
     if (back == NULL) {
@@ -2285,6 +2360,12 @@ static struct sftp_cmd_lookup {
 	    "  when you are not already connected to a server.\n",
 	    sftp_cmd_open
     },
+    {
+	"proxy", TRUE, "set a proxy",
+	    " <type> <host> [ <port> <user> ] <pass>\n"
+	    "  Type is 0 for no proxy, 1 for HTTP/1.1 and 2 for SOCKS5\n",
+	    sftp_cmd_proxy
+    },
     {
 	"put", TRUE, "upload a file from your local machine to the server",
 	    " [ -r ] [ -- ] <filename> [ <remote-filename> ]\n"
@@ -2948,7 +3029,9 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
     /*
      * Trim a colon suffix off the hostname if it's there.
      */
+    /* FZ: Doing so ruins ability to connect to IPv6 addresses
     cfg.host[strcspn(cfg.host, ":")] = '\0';
+    */
 
     /*
      * Remove any remaining whitespace from the hostname.
@@ -3148,6 +3231,9 @@ int psftp_init_utf8_locale()
  */
 int psftp_main(int argc, char *argv[])
 {
+    // FZ: Set proxy to none
+    cfg.proxy_type = PROXY_NONE;
+
     int i;
     int portnumber = 0;
     char *userhost, *user;
